//! \file lconfig.cc Реализация класса взаимодействия с конфигурацией
#include <fstream>
#include <iostream>
#include <sys/stat.h>
#include <sys/statfs.h>

#include "lconfig.h"
#include <libconfig.h++>

using namespace libconfig;

//! Конструктор класса по умолчанию
Lconfig::Lconfig()
{
    path=string();
}
//! Конструктор класса
/**
 * \param p путь к конфигурации
 */
Lconfig::Lconfig(string p):path(p){}
//! Деструктор класса
Lconfig::~Lconfig(){}

//! считываем информацию из конфигурационного файла
int Lconfig::readConfig()
{
    try
    {
       conf.readFile(path.c_str());
    }
    catch(const FileIOException &fioex)
    {
       std::cerr << "I/O error while reading config file!"<< std::endl;   
       return -1;
    }
    catch(const ParseException &pex)
    {
       std::cerr<< " Parse error at " << pex.getFile() << ":" << pex.getLine()
                << " - " << pex.getError() << std::endl; 
       return -1;
    }
    return 0;
}

//! записываем информацию в файл
int Lconfig::writeConfig()
{
    struct stat st;
    long f_size, ret;
    if(stat(path.c_str(), &st)==0)
    	f_size = st.st_size;
    else 
    	return -2;

    struct statfs sfs;
    ret=statfs(path.substr(0,path.rfind("/")).c_str(),&sfs);
    if(!ret){
    	if((f_size/sfs.f_bsize) >= sfs.f_bfree){ 				//Size ERROR
			std::cerr << "Ошибка сохранения файла! Нет свободного места на диске!" << endl;
			return -2;
        }
    }

    try
    {
       conf.writeFile(path.c_str());
    }
    catch(const FileIOException &fioex)
    {
       std::cerr << " I/O error while writing config file!" << endl;
       return -1;
    }
    return 0;
}

//! считывание строкового параметра
string Lconfig::readString(string param)
{
    string val="";
    try 
    {
      val = (const char*)conf.lookup(param.c_str());
    }
    catch(const SettingNotFoundException &nfex) 
    {
       std::cerr << "Error read "<<param<<"  from configuration file!" << endl;
       return ERR_STRING;
    }
    return val;
}

//! запись строкового параметра
int Lconfig::writeString(string param, string value)
{
    try 
    {
       conf.lookup(param.c_str()) = value;
    }
    catch(const SettingNotFoundException &nfex) 
    {
       std::cerr << "Error write "<<param<<" in configuration file!" << endl;
       return -1;
    }
    return 0;
}

//! считывание параметра типа int
int Lconfig::readInt(string param)
{
    int val=0;
    try 
    {
      val = conf.lookup(param.c_str());
    }
    catch(const SettingNotFoundException &nfex) 
    {
       std::cerr << "Error read "<<param<<" from configuration file!" << endl;
       return -1;
    }
    return val;
}

//! запись параметра типа int
int Lconfig::writeInt(string param, int value)
{
    try 
    {
       conf.lookup(param.c_str()) = value;
    }
    catch(const SettingNotFoundException &nfex) 
    {
       std::cerr << "Error write "<<param<<" in configuration file!" << endl;
       return -1;
    }
    return 0;
}

//! считывание параметра типа long int
long long Lconfig::readLongInt(string param)
{
    long long val=0;
    try 
    {
      val = conf.lookup(param.c_str());
    }
    catch(const SettingNotFoundException &nfex) 
    {
       std::cerr << "Error read "<<param<<" from configuration file!" << endl;
       return -1;
    }
    return val;
}

//! запись параметра типа long int
int Lconfig::writeLongInt(string param, long long value)
{
    try 
    {
       conf.lookup(param.c_str()) = value;
    }
    catch(const SettingNotFoundException &nfex) 
    {
       std::cerr << "Error write "<<param<<" in configuration file!" << endl;
       return -1;
    }
    return 0;
}

//! считывание параметра типа int через функцию lookupValue()
bool Lconfig::readIntLookup(string param, int& value)
{
    try 
    {
      return conf.lookupValue(param.c_str(), value);
    }
    catch(const SettingNotFoundException &nfex) 
    {
       std::cerr << "Error read "<<param<<" from configuration file!" << endl;
       return false;
    }
    return false;
}
