#include "ipcmedia.h"

#include <iostream>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <sys/ioctl.h>

IPCMedia::IPCMedia()
{

}

IPCMedia::~IPCMedia()
{

}

int IPCMedia::send(std::string receiver, std::string sender, int type, unsigned char* data, unsigned long len)
{
	int rc;
    IPCPacket* pkt = IPCPacket::Create(type, (char*)sender.c_str(), data, len);
    rc = send(receiver,sender,pkt);
    delete pkt;
    return rc;
}

int IPCMedia::wrapsend(std::string receiver, std::string sender, int type, unsigned char* data, unsigned long len)
{
    return IPCMedia::send(receiver, sender, type, data, len);
}

int IPCMedia::send(std::string receiver, std::string sender, IPCPacket* pkt)
{
	int s,rc;

	if(pkt==NULL)
	{
		std::cerr<<"IPCMedia::send("<<sender<<" > "<<receiver<<"): error NULL pkt "<<std::endl;
		return -1;
	}

	s = socket(AF_LOCAL, SOCK_DGRAM, 0);
	if(s == -1)
	{
		std::cerr<<"IPCMedia::send("<<sender<<" > "<<receiver<<", type:"<<pkt->getType()<<", len:"<<pkt->getDataLen()<<": error creating socket"<<std::endl;
		return -1;
	}

	rc = send(s,receiver,sender,pkt);
	close(s);

	return rc;
}

int IPCMedia::wrapsend(std::string receiver, std::string sender, IPCPacket* pkt)
{
    return IPCMedia::send(receiver, sender, pkt);
}

int IPCMedia::send(int s, std::string receiver, std::string sender, IPCPacket* pkt)
{
	int rc;
	struct sockaddr_un addr;

	if(pkt==NULL)
	{
		std::cerr<<"IPCMedia::send("<<sender<<" > "<<receiver<<"): error NULL pkt "<<std::endl;
		return -1;
	}

	addr.sun_family = AF_UNIX;
	strcpy(addr.sun_path, receiver.c_str());

	rc = sendto(s, pkt->getBuffer(), pkt->getSize(), 0, (struct sockaddr *)&addr, (socklen_t)sizeof(addr));

	if (rc  != (int) pkt->getSize())
	{
		std::cerr<<"IPCMedia::send("<<sender<<" > "<<receiver<<", type:"<<pkt->getType()<<", len:"<<pkt->getDataLen()<<": sendto result:"<<rc<<std::endl;
		return -1;
	}
	std::cerr<<"IPCMedia::send("<<sender<<" > "<<receiver<<", type:"<<pkt->getType()<<", len:"<<pkt->getDataLen()<<":OK("<<rc<<")"<<std::endl;
	return 0;
}

int IPCMedia::wrapsend(int s, std::string receiver, std::string sender, IPCPacket* pkt)
{
    return IPCMedia::send(s, receiver, sender, pkt);
}

extern "C" IPCMedia* getMediaFactory() {
  return new IPCMedia;
}

