//! \file psyncconfig.h Объявление класса основной формы приложения
#ifndef PSYNCCONFIG_H
#define PSYNCCONFIG_H

#include <QDialog>
#include <QMouseEvent>
#include <QKeyEvent>
#include <QFileDialog>
#include <QDir>
#include <QLineEdit>
#include <QSocketNotifier>
#include <QProcess>
#include <QDateTime>
#include <QMessageBox>
#include <QCursor>
#include <kuser.h>
#include <klocale.h>
#include <kglobal.h>
#include <kfiledialog.h>
#include <kurl.h>

#include <kstandardaction.h>
#include <qdebug.h>
#ifdef SYNC_DB
#include "libsync_db.h"
#endif
#include "framelogin.h"
#include "ipcpacket.h"
#include "ipc_interface.h"
#include "unixsocket.h"
#include "ui_psync_config.h"
#ifndef BUILD_APP
#include <kcmodule.h>
#endif
namespace Ui {
    class PSyncConfig;    
}
//! Класс приложения или плагина в зависимости от определенного флага при компиляции
class PSyncConfig
#ifdef BUILD_APP
 : public QDialog,  public Ui::PSyncConfig
#else
 : public KCModule, public Ui::PSyncConfig
#endif
{
    Q_OBJECT
public:
#ifdef BUILD_APP
    explicit PSyncConfig();
#else
    explicit PSyncConfig(QWidget *parent, const QVariantList &args);
#endif
    ~PSyncConfig();    


protected:
    bool eventFilter(QObject *obj, QEvent *ev);


private slots:
    void syncIntervalsChanged(int index);
    void setLineEditText(int index);
    void setComboBoxEnable(int state);
    void callFileDialog();
    void openAccountInBrowser();
    void syncronizeStartStop();
    void receiveNotification();
    void onLoginButton();
    void settingsChanged();
    void sendSyncronizeIs();
    void showTrayIconChanged();
    void showStartEndNotifyChanged();
    void showFileNotifyChanged();
    void conflictFileToSave(const QString &filename, unsigned int timer);
    void refuseSaveFile();

    void on_m_button_logout_clicked();

    void on_m_btn_addobject_clicked();

    void on_m_btn_delobject_clicked();

    void on_tabWidget_currentChanged(int index);

private:
    inline void initComboBox();
    inline void createComboBoxsLineEdit();
    inline void readSettings();        
    inline void saveSettings();
    inline void syncSettingsInstance();
    inline void initSocket();
    inline void setUserPixmap();
    inline void setLastSyncNote(const QDateTime &lastsync);// = QDateTime());
    inline void initFrameLogin();

    void clearAuthData();
    void updateSyncInfo();
    void checkPassword(QString login, QString password);
    void showPasswordRequest();
    void showRegularView();
    void updateQuotaInfo();
    void setQuoteTotal(unsigned long long bytes);
    void setQuoteUsed(unsigned long long bytes);
    void initMargins();
    void sendSettingsChangedNotify();    
    unsigned long fromIndexToSec(int index);
    int fromSecToIndex(unsigned long Sec);

private:
    void updateRegularView();
#ifdef SYNC_DB
    Libsync_db *db;
#endif
    Ui::PSyncConfig *ui;                    //!< Главный графический объект приложения
    QLineEdit *m_ln;                        //!< Текстовое поле с названием интервала периодической синхронизации
    QLineEdit *m_svpnt;                     //!< Текстовое поле с названием точки сохранения синхронизации
    QPushButton *m_login_button;            //!< Кнопка проверки введенных аутентификационных данных
    QPushButton *m_create_account_button;   //!< Кнопка создания нового аккаутна пользователя
    FrameLogin *frmlgn;                     //!< Объект формы ввода аутентификационных данных
    QMessageBox *fileToSave;
    QTimer *notify_timer;

    bool nospace;
    unsigned int m_timer;
    bool m_sync_is_running;                 //!< Состояние синхронизации

    UnixSocket *m_us;                       //!< Объект класса UnixSocket из библиотеки libpsync
    QSocketNotifier *m_sn;                  //!< Объект для поддержки мониторинга активности файлового дескриптора
    Qt::WindowFlags wf;                     //!< Флаги окна

    QDateTime m_datetime_lastsync;          //!< Время последней синхронизации

    QString m_login;                        //!< Логин пользователя
    QString m_password;                     //!< Пароль пользователя
};

#endif // PSYNCCONFIG_H
