//! \file psyncconfig.cpp Реализация класса основной формы приложения
#include "../libpsync/include/sync_global.h"
#include "../psyncnotify/include/ipc_interface.h"
#include "../syncd/include/ipc_interface.h"

#include "psyncconfig.h"
#include "config.h"
//#include "ui_psyncconfig.h"

#include "ipc_interface.h"
#include "ipc_interface_local.h"

#include <ipcmedia.h>
#include <iostream>
#include <kio/scheduler.h>
// include notifier programm IPC API

#include "../liblcfg/include/lconfig.h"

#include <QDesktopWidget>
#ifndef BUILD_APP
#include <kpluginfactory.h>
#endif

#define DEFAULT_FOLDER_STR_SEPARATOR ": "

#define _SERVICE_NAME 				i18n(GLOBAL_SERVICE_PRODUCT_NAME)                               //!< Общее название продукта
#define _PROGRAMM_NAME 				i18n("%1 configuration",QString(GLOBAL_SERVICE_PRODUCT_NAME))   //!< Название приложения

#ifndef BUILD_APP
K_PLUGIN_FACTORY(factory, registerPlugin<PSyncConfig>();)   //!< Регистрация фабрики плагина
K_EXPORT_PLUGIN(factory("psyncconfig", "psyncconfig"))      //!< Экспорт главного объекта плагина
#endif

#ifdef BUILD_APP
//!< Конструктор класса приложения
PSyncConfig::PSyncConfig() :
    ui(new Ui::PSyncConfig)
#else
//! Конструктор класса плагина
/**
  * \param args не исплозьзуется
  * \param parent объект-родитель
  * Инициализация графических элементов приложения
  */
PSyncConfig::PSyncConfig(QWidget *parent, const QVariantList &args) :
KCModule(factory::componentData(),parent,args),
    ui(new Ui::PSyncConfig)
#endif
{
    if(!QFile::exists(PSYNCD_IPC_NAME)) {
        QMessageBox::critical(this, _SERVICE_NAME, i18n("MandrivaSync service is not running. Please contact your system administrator."), QMessageBox::Ok);
        exit(-1);
    }

#ifndef BUILD_APP
    Buttons btn = 0;
    setButtons(btn);
#endif

    m_login = "";
    m_password = "";
    m_sync_is_running=false;
    nospace = false;
    ui->setupUi(this);

    initFrameLogin();
    initComboBox();
    createComboBoxsLineEdit();

    //ui->comboBox_syncronize_interval->installEventFilter(this);
    connect(ui->checkBox_is_syncronized, SIGNAL(stateChanged(int)), this, SLOT(setComboBoxEnable(int)));
    //connect(ui->pushButton_change_folder, SIGNAL(clicked()), this, SLOT(callFileDialog()));
    connect(ui->pushButton_parameters, SIGNAL(clicked()), this, SLOT(openAccountInBrowser()));

    syncSettingsInstance();
    readSettings();

    if(QFile::exists(PSYNCCONFIG_IPC_NAME))
    {
        int rc = IPCMedia::send(PSYNCCONFIG_IPC_NAME,PSYNCCONFIG_IPC_NAME,PSYNCCONFIG_ACTIVATE_WINDOW);
        qDebug("STARTRC=%d",rc);
        if(rc==0) exit(0);
        QFile::remove(PSYNCCONFIG_IPC_NAME);
    }

    setLineEditText(ui->comboBox_syncronize_interval->currentIndex());
    setUserPixmap();

    initSocket();
    connect(ui->pushButton_syncronize, SIGNAL(clicked()), this, SLOT(syncronizeStartStop()) );

    //initMargins();
    connect(ui->m_timer_spin, SIGNAL(valueChanged(int)), this, SLOT(settingsChanged()));
    connect(ui->checkBox_is_syncronized, SIGNAL(stateChanged(int)), this, SLOT(settingsChanged()));
    connect(ui->checkBox_autosync, SIGNAL(stateChanged(int)), this, SLOT(settingsChanged()));
    connect(ui->checkBox_is_syncronized, SIGNAL(stateChanged(int)), this, SLOT(sendSyncronizeIs()));

    connect(ui->comboBox_syncronize_interval, SIGNAL(currentIndexChanged(int)), this, SLOT(settingsChanged()));
    connect(ui->m_cb_showTrayIcon, SIGNAL(stateChanged(int)), this, SLOT(showTrayIconChanged()));
    connect(ui->m_cb_showStartEndNotifications, SIGNAL(stateChanged(int)), this, SLOT(showStartEndNotifyChanged()));
    connect(ui->m_cb_showFileNotifications, SIGNAL(stateChanged(int)), this, SLOT(showFileNotifyChanged()));
    
    ui->pushButton_change_folder->setVisible(false); // TODO: del default dir
    ui->label_help_note->setVisible(false);          // TODO: del default dir
    ui->label_save_folder_header->setVisible(false); // TODO: del default dir
    ui->pushButton_save->setVisible(false);
    ui->pushButton_restore->setVisible(false);
    ui->pushButton_clear->setVisible(false);
    ui->pushButton_new_save_point->setVisible(false);
    ui->comboBox_save_point->setVisible(false);
    ui->label_save_date->setVisible(false);
    ui->label_save_date_note->setVisible(false);
    ui->label_save_point_note->setVisible(false);
    ui->label_size->setVisible(false);
    ui->label_size_notr->setVisible(false);
    ui->label_new_save_point_note->setVisible(false);
    ui->label_name_save_point->setVisible(false);
    ui->lineEdit_name_save_point->setVisible(false);
    ui->line->setVisible(false);

    IPCMedia::send(PSYNCD_IPC_NAME, PSYNCCONFIG_IPC_NAME, PSYNCD_GET_SYNC_STATUS);

    qApp->installEventFilter(this);

    if (m_password.isEmpty())
    {
        showPasswordRequest();
    }
    else
    {
        checkPassword(m_login, m_password);
    };
    updateRegularView();

}
//! Деструктор класса
/**
  * Сохранение текущих настроек
  * Закрытие сокета
  * Удаление объектов UnixSocket и главного графического объекта
  */
PSyncConfig::~PSyncConfig()
{
    saveSettings();    
    m_us->finish();
    QFile::remove(PSYNCCONFIG_IPC_NAME);
#ifdef SYNC_DB
    delete db;      // TODO: DataBase
#endif
    delete ui;
    delete m_us;
}

/*void PSyncConfig::show()
{
	setWindowTitle(_PROGRAMM_NAME);
	QDialog::show();
}
*/

//! Функция инициализации формы ввода аутентификационных данных пользователя
void PSyncConfig::initFrameLogin()
{
    frmlgn = new FrameLogin(ui->tab_account);
    frmlgn->setFrameStyle(QFrame::Plain);        
    frmlgn->setVisible(false);    

    ui->tab_account->layout()->addWidget(frmlgn);
    m_login_button = frmlgn->getButtonLogin();
    m_create_account_button = frmlgn->getButtonCreateAccount();
    m_create_account_button->setAutoFillBackground(true);
    m_create_account_button->setStyleSheet("  border: none; color: blue; ");

    connect(m_login_button, SIGNAL(clicked()), this, SLOT(onLoginButton()));
    connect(m_create_account_button, SIGNAL(clicked()), this, SLOT(openAccountInBrowser()));
}

//! Изменение интервала периодической синхронизации
/**
  * \param index номер поля в графическом элементе
  */
void PSyncConfig::syncIntervalsChanged(int index)
{
    setLineEditText(index);
}

//! Установка текста для выбранного поля в графическом элементе ComboBox
/**
  * \param index номер поля в графическом элементе
  */
void PSyncConfig::setLineEditText(int index)
{        
    m_ln->setText(ui->comboBox_syncronize_interval->itemText(index));
}

//! Инициализация графического элемента ComboBox
void PSyncConfig::initComboBox()
{
    ui->comboBox_syncronize_interval->insertSeparator(ui->comboBox_syncronize_interval->count() - 1);
    connect(ui->comboBox_syncronize_interval, SIGNAL(activated(int)), this, SLOT(syncIntervalsChanged(int)));
    connect(ui->comboBox_syncronize_interval, SIGNAL(highlighted(int)), this, SLOT(setLineEditText(int)));
    ui->comboBox_syncronize_interval->setInsertPolicy(QComboBox::NoInsert);        
}

//! Создание и установка графического элемента LineEdit в ComboBox
void PSyncConfig::createComboBoxsLineEdit()
{
    m_ln = new QLineEdit();
    m_ln->setReadOnly(true);
    ui->comboBox_syncronize_interval->setLineEdit(m_ln);

    m_svpnt = new QLineEdit;
    m_svpnt->setReadOnly(true);
    ui->comboBox_save_point->setLineEdit(m_svpnt);
}

bool PSyncConfig::eventFilter(QObject *obj, QEvent *ev)
{
    if (ev->type() == QEvent::ApplicationActivate) {
        if (!(m_login.isEmpty()) && !(m_password.isEmpty())) {
#ifndef SYNC_DB
            // TODO: old version update sync files without DB
            IPCPacket* pkt = IPCPacket::Create(PSYNCD_GET_ALL_FILES_COMMAND, (char*)PSYNCCONFIG_IPC_NAME,0,0);

            QString receiver_address(PSYNCD_IPC_NAME);

            if (m_us->writeDatagram((const char*)pkt->getBuffer(), pkt->getSize(), receiver_address.toLocal8Bit().data()) < 0)
            {
                qDebug("IPC error with %s: type %lu",receiver_address.toLocal8Bit().data(),pkt->getType());
                //QMessageBox::critical(this, QString("Sync Client"), i18n("The program error occurred. Information about the process and since the last synchronization may be displayed incorrectly"), QMessageBox::Ok);
            }

            if (pkt) delete pkt;
#else
            // TODO: new version update sync files with DB
            ui->m_list_objects->clear();
            ui->m_list_objects->setSelectionMode(QAbstractItemView::ExtendedSelection);
            db->get_objects(expr, &table);
            db_table::iterator cur;
            for(cur = table.begin(); cur < table.end(); ++cur) {
                row = (db_row)*cur;
                db_row::iterator r;
                for(r = row.begin(); r != row.end(); ++r) {
                    if (QString::fromLocal8Bit((*r).first.c_str()) == "local_path")
                        ui->m_list_objects->addItem(QString::fromLocal8Bit((*r).second.c_str()));
                }
            }
#endif
        }
    }
    return QWidget::eventFilter(obj, ev);
}

/*
bool PSyncConfig::eventFilter(QObject *obj, QEvent *event)
{
    if ((obj == ui->comboBox_syncronize_interval) && (event->type() == QEvent::KeyPress))
    {
        QKeyEvent *ke = (QKeyEvent*)event;
        if (ke->key() == Qt::Key_Space)
        {
            QMouseEvent *me = new QMouseEvent(QEvent::MouseButtonPress,
            		QPoint(ui->comboBox_syncronize_interval->width() - 5, ui->comboBox_syncronize_interval->height() - 5),
            		Qt::LeftButton, Qt::LeftButton, Qt::NoModifier);
            QCoreApplication::postEvent(ui->comboBox_syncronize_interval, me);
        }
    }    


        return QDialog::eventFilter(obj, event);
}
*/

//! Установка состояния доступности графических элементов синхронизации
/**
  * \param state флаг состояния доступности
  */
void PSyncConfig::setComboBoxEnable(int state)
{
    qDebug("state = %d", state);
        ui->comboBox_syncronize_interval->setEnabled(state);
        ui->pushButton_syncronize->setEnabled(state);
	ui->checkBox_autosync->setEnabled(state);
}

//! Функция чтения настроек из файла конфигурации
void PSyncConfig::readSettings()
{
//    const int size_value = 512;
//    char value[size_value];
    bool correct = true;
    bool correct_quota = true;
    unsigned int interval = 0;

    Lconfig conf(GLOBAL_SERVICE_LCONFIG_DIR); //config object
    if(conf.readConfig()!=0)  std::cerr <<__FILE__<<" "<<__LINE__<< " Error working with config!"<< std::endl; 

    int sync_is = conf.readInt(LSETTINGS_SYNC_IS);
    if(sync_is != -1)
       ui->checkBox_is_syncronized->setCheckState((Qt::CheckState)sync_is);
    else 
        correct = false;

    interval = conf.readInt(LSETTINGS_SYNC_INTERVAL);
    ui->comboBox_syncronize_interval->setCurrentIndex(fromSecToIndex(interval));
/*
    QString def_folder;
    string d_folder =  conf.readString(LSETTINGS_DEFAULT_FOLDER);
    if(d_folder != ERR_STRING)
    {
      if(d_folder == "")
          def_folder = QDir::homePath()+"/.sync-unresolved";
      else 
          def_folder = QString::fromLocal8Bit(d_folder.c_str());
    }
    else  correct = false;

    ui->label_current_folder->setText(ui->label_current_folder->text() + def_folder);
*/
    ui->label_current_folder->setVisible(false); // TODO: del default dir

    int time=0;
    time = conf.readInt(LSETTINGS_LAST_SYNC_TIME);
    if(time != -1 && time != 0)
    {
        QDateTime dt;
        dt.setTime_t(time);
        setLastSyncNote(dt);
    }

    unsigned long mode=1;
    mode = conf.readInt(LSETTINGS_SYNC_MODE);
    if(mode==2)
	    ui->checkBox_autosync->setChecked(true);
    else
	    ui->checkBox_autosync->setChecked(false);

    int val;
    val = conf.readInt(LSETTINGS_SHOW_TRAY);
    if(val != -1)  ui->m_cb_showTrayIcon->setCheckState((Qt::CheckState)val);
    else {
        ui->m_cb_showTrayIcon->setChecked(true); // TODO: TrayIcon
        correct = false;
    }

    val = conf.readInt(LSETTINGS_SHOW_NOTIFY_START_END);
    if(val != -1)  ui->m_cb_showStartEndNotifications->setCheckState((Qt::CheckState)val);
    else {
        ui->m_cb_showStartEndNotifications->setChecked(true);
        correct = false;
    }

    val = conf.readInt(LSETTINGS_SHOW_NOTIFY_FILE);
    if(val != -1)  ui->m_cb_showFileNotifications->setCheckState((Qt::CheckState)val);
    else {
        ui->m_cb_showFileNotifications->setChecked(true);
        correct = false;
    }

    long long quota = 0;
    long long quota_u = 0;
    quota = conf.readLongInt(LSETTINGS_QUOTA_TOTAL);
    if(quota != -1)
        setQuoteTotal(quota);
    else
        correct_quota = false;

    quota_u = conf.readLongInt(LSETTINGS_QUOTA_USED);
    if(quota_u != -1)
        setQuoteUsed(quota_u);
    else
        correct_quota = false;

    if (correct_quota)
        ui->progressBar->setToolTip(i18n("Used %1 Mb. of available %2 Mb.", quota_u / 1000 / 1000, quota / 1000 / 1000));

    string str = conf.readString(LSETTINGS_LOGIN);
    if (str != ERR_STRING)
    {
	m_login = str.c_str();
	ui->label_account->setText(str.c_str());
	frmlgn->setLogin(str.c_str());
    }
    else return;

    str = conf.readString(LSETTINGS_PASSWORD);
    if (str != ERR_STRING)  m_password = QVariant(str.c_str()).toString();
    else return;

    unsigned int timer=0;
    timer = conf.readInt(LSETTINGS_CONFLICT_TIMER);
    if((int)timer != -1) {
        ui->m_timer_spin->setValue(timer);
        m_timer = timer;
    }
    else {
        ui->m_timer_spin->setValue(10); // TODO: timer
        m_timer = 10;
    }

    if (!correct)
        QMessageBox::critical(this, _SERVICE_NAME, i18n("Error reading configuration"), QMessageBox::Ok);
}

//! Функция сохранения настроек в файл конфигурации
void PSyncConfig::saveSettings()
{
//	int rc;
    bool is_save = true;
    Lconfig conf(GLOBAL_SERVICE_LCONFIG_DIR); //config object
    if(conf.readConfig()!=0)   std::cerr <<__FILE__<<" "<<__LINE__<< " Error working with config!"<< std::endl; 

    if(conf.writeInt(LSETTINGS_SYNC_IS, ui->checkBox_is_syncronized->checkState()) != 0) is_save = false;
    if(conf.writeInt(LSETTINGS_SHOW_TRAY, ui->m_cb_showTrayIcon->checkState()) != 0) is_save = false;
    if(conf.writeInt(LSETTINGS_SHOW_NOTIFY_START_END, ui->m_cb_showStartEndNotifications->checkState()) != 0) is_save = false;
    if(conf.writeInt(LSETTINGS_SHOW_NOTIFY_FILE, ui->m_cb_showFileNotifications->checkState()) != 0) is_save = false;

    unsigned long interval = fromIndexToSec(ui->comboBox_syncronize_interval->currentIndex());
    if (conf.writeInt(LSETTINGS_SYNC_INTERVAL, interval) != 0) is_save = false;

    unsigned long mode = ui->checkBox_autosync->isChecked()?2:1;
    if (conf.writeInt(LSETTINGS_SYNC_MODE, mode) != 0) is_save = false;
/*
    QStringList sl = ui->label_current_folder->text().split(QString(DEFAULT_FOLDER_STR_SEPARATOR));

    string str =  (const char*)sl[1].toLocal8Bit().data();
    if (conf.writeString(LSETTINGS_DEFAULT_FOLDER, str) != 0) is_save = false;
*/
    string str;
    str =  (const char*)QVariant(m_login).toString().toLocal8Bit().data();
    if(conf.writeString(LSETTINGS_LOGIN, str) != 0) is_save = false;

    str =  (const char*)QVariant(m_password).toString().toLocal8Bit().data();
    if(conf.writeString(LSETTINGS_PASSWORD, str) != 0) is_save = false;

    unsigned int timer = ui->m_timer_spin->value();
    if (conf.writeInt(LSETTINGS_CONFLICT_TIMER, timer) != 0) is_save = false;

    if(conf.writeConfig()!=0)
    {
        std::cerr <<__FILE__<<" "<<__LINE__<< " Error working with config!"<< std::endl;
        is_save = false;
    }

    if(!is_save) {
        if (!nospace) {
            QMessageBox::critical(this, _SERVICE_NAME, i18n("Error saving settings"), QMessageBox::Ok);
            nospace = true;
        }
    } else
        nospace = false;

    sendSettingsChangedNotify();
}

//! Функция проверки логина и пароля
/**
  * \param login логин пользователя
  * \param password пароль пользователя
  */
void PSyncConfig::checkPassword(QString login, QString password)
{
    qDebug("login = %s, password = %s", login.toLocal8Bit().data(), password.toLocal8Bit().data());

    if ((login.isEmpty()) || (password.isEmpty()))
    {
        QMessageBox::warning(this, _SERVICE_NAME, i18n("Need to enter your login and password of nonzero length"), QMessageBox::Ok);
        return ;
    }

	QString info = login + ":" + password;
	IPCPacket* pkt = IPCPacket::Create(REQUEST_LOGIN_AND_PASSWORD, (char*)CONFIG_PROGRAM_NAME, (unsigned char*)info.toLocal8Bit().data(), (unsigned long)info.length());

	QString receiver_address(PSYNCD_IPC_NAME);
	if (m_us->writeDatagram((const char*)pkt->getBuffer(), pkt->getSize(), receiver_address.toLocal8Bit().data()) < 0)
	{
		qDebug("IPC error with %s: type %lu",receiver_address.toLocal8Bit().data(),pkt->getType());
		//QMessageBox::critical(this, i18n("Sync Client"), i18n("The program error occurred. Information about the process and since the last synchronization may be displayed incorrectly"), QMessageBox::Ok);
	}

	if (pkt) delete pkt;

}

//! Отображение формы запроса аутентификационных данных пользователя
void PSyncConfig::showPasswordRequest()
{
    m_password = "";
    m_login = "";
    ui->tab_objects->setEnabled(false);
    ui->tab_settings->setEnabled(false);
    ui->frame_save_point->setVisible(false);
    ui->tab_interface->setEnabled(false);
    frmlgn->setLogin("");
    frmlgn->setPassword("");
    frmlgn->show();
}

//! Не используется
void PSyncConfig::syncSettingsInstance()
{

}

//! Функция вызова файлового диалога для выбора папки "по умолчанию"
void PSyncConfig::callFileDialog()
{
        //QStringList pe = QProcess::systemEnvironment();

        //QStringList sl = ui->label_current_folder->text().split(QString(DEFAULT_FOLDER_STR_SEPARATOR));

    IPCMedia::send(PSYNCD_IPC_NAME, PSYNCCONFIG_IPC_NAME, PSYNCD_FILEDIALOG);
    notify_timer->stop();

    QString sf;     // TODO: del this vriable and to do with QStringList

    QFileDialog *dialog = new QFileDialog(this, i18n("Folder to save files by default"));
    dialog->setAcceptMode(QFileDialog::AcceptOpen);
    dialog->setDirectory(QDir::homePath());
    dialog->setFileMode(QFileDialog::Directory);
    dialog->setOption(QFileDialog::DontUseNativeDialog,true);
    dialog->setLabelText(QFileDialog::FileName, i18n("Name: "));
    dialog->setLabelText(QFileDialog::FileType, i18n("Filter: "));
    dialog->setLabelText(QFileDialog::LookIn, i18n("Places")); // TODO: Look in
    dialog->setLabelText(QFileDialog::Accept, i18n("OK"));
    dialog->setLabelText(QFileDialog::Reject, i18n("Cancel"));

    QStringList fileNames;
    if (dialog->exec()) fileNames = dialog->selectedFiles();
        if (!fileNames.isEmpty())
            sf = fileNames[0];
        else {
            refuseSaveFile();
            return;                 // TODO: answer only to daemon
        }

    QFileInfo fi(sf);

    while (!fi.isWritable()) {
        QMessageBox::critical(this, _SERVICE_NAME, i18n("It is impossible to write down files in a folder. Please, choose a writable folder."), QMessageBox::Ok);
        dialog->selectFile(QDir::homePath());
        fileNames.clear();
        if (dialog->exec()) fileNames = dialog->selectedFiles();
            if (!fileNames.isEmpty())
                sf = fileNames[0];
            else {
                refuseSaveFile();
                return;             // TODO: answer only to daemon
            }
        fi.setFile(sf);
    }

    qDebug() << "timer = " << m_timer;
    qDebug() << "dir ame = "  << sf;
    IPCMedia::send(PSYNCD_IPC_NAME, PSYNCCONFIG_IPC_NAME, PSYNCD_CONFLICT_SAVE_FILE, (unsigned char*)(sf.toLocal8Bit().data()), sf.size());
}

//! Функция открытия страницы регистрации нового аккаунта пользователя в браузере
void PSyncConfig::openAccountInBrowser()
{
    QString program = "/usr/bin/www-browser";
    QStringList arguments;
    arguments << "https://www.mandrivasync.com";
    QProcess* process = new QProcess(this);
    process->start(program, arguments);
}

///
/// Syncronisation start/stop button slot
///

//! Функция начала и завершения синхронизации
void PSyncConfig::syncronizeStartStop()
{
    IPCPacket * pkt = NULL;

    QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
    ui->pushButton_syncronize->setEnabled(false);
    ui->checkBox_autosync->setEnabled(false);
    ui->comboBox_syncronize_interval->setEnabled(false);
    ui->checkBox_is_syncronized->setEnabled(false);

    if (m_sync_is_running)
    {
        pkt = IPCPacket::Create(PSYNCD_SYNC_STOP, (char*)PSYNCCONFIG_IPC_NAME,0,0);
    }
    else
    {
        pkt = IPCPacket::Create(PSYNCD_SYNC_START, (char*)PSYNCCONFIG_IPC_NAME,0,0);
    }

	if (m_us->writeDatagram((const char*)pkt->getBuffer(), pkt->getSize(), PSYNCD_IPC_NAME) < 0)
    {
		QApplication::restoreOverrideCursor();
		m_sync_is_running=!m_sync_is_running;
		qDebug("IPC error with %s: type %lu",PSYNCD_IPC_NAME,pkt->getType());
        //QMessageBox::critical(this, QString("Sync Client"), i18n("The program error occurred. Information about the process and since the last synchronization may be displayed incorrectly"), QMessageBox::Ok);
    }
    
    if(pkt) delete pkt;

    //updateRegularView();
}

///
/// IPC Socket creation
///

//! Функция инициализации сокета
void PSyncConfig::initSocket()
{
    m_us = new UnixSocket();
    if (m_us->init())
    {
        qDebug("sockfd = %d", m_us->getSockfd());
        QString self_address(PSYNCCONFIG_IPC_NAME);
        QFileInfo fi(PSYNCCONFIG_IPC_NAME);
        if (!fi.absoluteDir().exists())
        	fi.absoluteDir().mkdir(PSYNCCONFIG_IPC_NAME);

        if (m_us->bind(self_address.toLocal8Bit().data()))
        {
            m_sn = new QSocketNotifier(m_us->getSockfd(), QSocketNotifier::Read);
            connect(m_sn , SIGNAL(activated(int )), this, SLOT(receiveNotification()));
        }
        else
        {
            qDebug("Error of local socket bind");//TODO//
            QMessageBox::critical(this, _SERVICE_NAME, i18n("The program error occurred. Information about the process and since the last synchronization may be displayed incorrectly"), QMessageBox::Ok);
        }
    }
    else
    {
        qDebug("Error of local socket init");
        QMessageBox::critical(this, _SERVICE_NAME, i18n("The program error occurred. Information about the process and since the last synchronization may be displayed incorrectly"), QMessageBox::Ok);
    }

    // TODO: // m_sync_is_running = false;
}

//! Слот, связанный с кнопкой запроса на проверку аутентификационных данных, введенных пользователем
void PSyncConfig::onLoginButton()
{
    m_login = frmlgn->getLogin();
    m_password = frmlgn->getPassword();
    checkPassword(m_login, m_password);
    if (!(m_login.isEmpty()) && !(m_password.isEmpty())){
        QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
    }
}

//! Слот, связанный с кнопкой, отвечающей за разлогинивание пользователя
void PSyncConfig::on_m_button_logout_clicked()
{
    int ret = QMessageBox::question(this, _SERVICE_NAME, i18n("After log out current account information will be lost!\nPlease ensure you remember login and password.\nDo you really want to log out?"),QMessageBox::Yes|QMessageBox::No);

    if(ret == QMessageBox::No) return;

    clearAuthData();
    ui->m_list_objects->clear();
    ui->checkBox_is_syncronized->setChecked(false);

    saveSettings();
    IPCMedia::send(PSYNCD_IPC_NAME,PSYNCCONFIG_IPC_NAME,PSYNCD_SETTINGS_CHANGED);

    showPasswordRequest();
}

//! Обработка сообщений от модулей psyncd и psyncconfig
void PSyncConfig::receiveNotification()
{    
    IPCPacket *m_pkt = NULL;
    QString res;
    QStringList all_objects;

#ifdef SYNC_DB
    db_table table;
    db_expr expr;
    db_row row;
#endif

    int total_bytes = m_us->bytesAvailable();
    if (m_pkt)
        return;

    m_pkt = new IPCPacket();
    m_us->readDatagram((char*)m_pkt->readBuffer(total_bytes), total_bytes);    

    if (m_pkt->IsValid())
    {
        switch (m_pkt->getType())
        {
        	case PSYNCCONFIG_SYNC_START:
                         ui->pushButton_syncronize->setEnabled(true);
        	case PSYNCD_ANS_SYNC_STATUS_ONLINE:
                                m_sync_is_running = true;               // TODO: more less :(
				QApplication::restoreOverrideCursor();
				updateRegularView();
				break;
                case PSYNCNOTIFY_HTTP_ERROR:
                case PSYNCCONFIG_SYNC_STOP:
                         ui->pushButton_syncronize->setEnabled(true);
                case PSYNCD_ANS_SYNC_STATUS_OFFLINE:
                                m_sync_is_running = false;              // TODO: more less :(
				QApplication::restoreOverrideCursor();
                                updateRegularView();
				break;
            case PSYNCCONFIG_ACTIVATE_WINDOW:
            	{
                     #ifdef BUILD_APP
                         QRect d = QApplication::desktop()->screenGeometry();
                         QRect w = geometry();
                         move((d.width()-w.width())/2,(d.height()-w.height())/2);
                     #endif
                     setWindowState(Qt::WindowStates(Qt::WindowActive));
                     raise();
                     activateWindow();
            	}
                break;
            case PSYNCNOTIFY_CONFLICT_SAVE_FILE:
                setWindowState(Qt::WindowStates(Qt::WindowActive));
                raise();
                activateWindow();
                readSettings();
                res = QString::fromLocal8Bit(QByteArray((const char*)m_pkt->getData(),m_pkt->getDataLen()));
                qDebug() << "res = " << res;
                conflictFileToSave(res, m_timer);
                break;
            case PSYNCNOTIFY_FILERECV_NOSPACE:
                nospace = true;
            case PSYNCNOTIFY_FILEDIALOG_STATE:
                PSyncConfig::setEnabled(!PSyncConfig::isEnabled());
                break;
            case SETTINGS_CLOSE_COMMAND:
                close();
                break;
            case PSYNCCONFIG_ACCOUNT_ERROR:
                QApplication::restoreOverrideCursor();
                clearAuthData();
            	QMessageBox::critical(this, _SERVICE_NAME, i18n("Error connecting to server"), QMessageBox::Ok);
                //showPasswordRequest();
                break;
 	    case PSYNCCONFIG_ACCOUNT_INVALID:
		QApplication::restoreOverrideCursor();
		QMessageBox::critical(this, _SERVICE_NAME, i18n("Account name or password is invalid"), QMessageBox::Ok);
		showPasswordRequest();
		break;
 	    case PSYNCCONFIG_ACCOUNT_VALID:
            #ifdef SYNC_DB
                db = new Libsync_db(m_login.toStdString()); // TODO: check existing DataBase
                // TODO: new version of function with DB
                if (db->is_exist_db()) {
                    ui->m_list_objects->clear();
                    ui->m_list_objects->setSelectionMode(QAbstractItemView::ExtendedSelection);
                    db->get_objects(expr, &table);
                    db_table::iterator cur;
                    for(cur = table.begin(); cur < table.end(); ++cur) {
                        row = (db_row)*cur;
                        db_row::iterator r;
                        for(r = row.begin(); r != row.end(); ++r) {
                            if (QString::fromLocal8Bit((*r).first.c_str()) == "local_path")
                                ui->m_list_objects->addItem(QString::fromLocal8Bit((*r).second.c_str()));
                    }
                } else {
                    row["unic_name TEXT PRIMARY KEY"]="0";
                    row["local_path TEXT"] = "0";
                    row["sTag TEXT"] = "0";
                    row["sync_time TEXT"] = "0";
                    row["mode TEXT"] = "0";
                    row["sync_point TEXT"] = "0";
                    row["flags TEXT"] = "0";
                    db->create_db(row);
                }
            #endif
		showRegularView();
		break;
            case PSYNCD_OBJECTS_LIST:
#ifndef SYNC_DB
            // TODO: old version of function without DB
                if (!(m_login.isEmpty()) && !(m_password.isEmpty())) {
                    res = QString::fromLocal8Bit((QByteArray((const char*)m_pkt->getData(),m_pkt->getDataLen())));
                    //if(res.length()==0) break;
                    all_objects = res.split("\n",QString::SkipEmptyParts);
                    ui->m_list_objects->clear();
                    ui->m_list_objects->setSelectionMode(QAbstractItemView::ExtendedSelection);
                    ui->m_list_objects->addItems(all_objects);
                    updateRegularView();
                }
#endif
                break;

            default:
                qDebug("Unknown type of message. The number of message is %d", (int)m_pkt->getType());
        }
    }

    if (m_pkt)
        delete m_pkt;
}

//! Функция обновления информации о времени синхронизации
void PSyncConfig::updateSyncInfo()
{
    unsigned int time=0;

    Lconfig conf(GLOBAL_SERVICE_LCONFIG_DIR); //config object
    if(conf.readConfig()!=0)  std::cerr <<__FILE__<<" "<<__LINE__<< " Error working with config!"<< std::endl; 
    time = conf.readInt(LSETTINGS_LAST_SYNC_TIME);

    if((int)time != -1 && (int)time != 0)
    {
        QDateTime dt;
        dt.setTime_t(time);
        setLastSyncNote(dt);
    }
}

//! Функция обновления текущего состояния доступности всех графических элементов приложения psyncconfig
void PSyncConfig::updateRegularView()
{
    if(m_password=="") return;
    
    if (!m_sync_is_running)
    {
    	ui->pushButton_syncronize->setText(i18n("Synchronize now"));
        ui->checkBox_is_syncronized->setEnabled(true);
        ui->m_button_logout->setEnabled(true);
        if (ui->checkBox_is_syncronized->isChecked()
                && ui->checkBox_is_syncronized->isEnabled()) {
            ui->comboBox_syncronize_interval->setEnabled(true);
            ui->checkBox_autosync->setEnabled(true);
        }
        ui->pushButton_change_folder->setEnabled(true);
        ui->tab_objects->setEnabled(true);
    }
    else
    {
    	ui->pushButton_syncronize->setText(i18n("Stop synchronization"));
        ui->checkBox_is_syncronized->setEnabled(false);
        ui->comboBox_syncronize_interval->setEnabled(false);
        ui->checkBox_autosync->setEnabled(false);
        ui->pushButton_change_folder->setEnabled(false);
        ui->m_button_logout->setEnabled(false);
        ui->tab_objects->setEnabled(false);
    }
    updateSyncInfo();
    updateQuotaInfo();
}

//! Функция установки пользовательской иконки, определенной в системе
void PSyncConfig::setUserPixmap()
{
    KUser *user = new KUser();
    QString user_avatar_path = user->faceIconPath();
    qDebug("path = %s", user_avatar_path.toLocal8Bit().data());

    if (!user_avatar_path.isEmpty())
        ui->label_image->setPixmap(QPixmap(user->faceIconPath()).scaled(QSize(64,64)));
}

//! Функция установки/отображения времени последней синхронизации
/**
  * \param lastsync время последней синхронизации
  */
void PSyncConfig::setLastSyncNote(const QDateTime &lastsync)
{
    KLocale *aLocale = KGlobal::locale();

    if (lastsync.isNull())
        ui->label_last_syncronization->setText(i18n("The last synchronization: ") + aLocale->formatDateTime(QDateTime::currentDateTime(), KLocale::FancyShortDate).toLower());
    else
        ui->label_last_syncronization->setText(i18n("The last synchronization: ") + aLocale->formatDateTime(lastsync, KLocale::FancyShortDate).toLower());
}

//! Функция отображения формы приложения psyncconfig
void PSyncConfig::showRegularView()
{
    QApplication::restoreOverrideCursor();
    frmlgn->setVisible(false);
    ui->frame_save_point->setVisible(true);
    ui->label_account->setText(m_login);
    ui->tab_objects->setEnabled(true);
    ui->tab_settings->setEnabled(true);
    ui->tab_interface->setEnabled(true);
    updateRegularView();
}

//! Функция обновления информации в квоте
void PSyncConfig::updateQuotaInfo()
{
    long long quota = 0;
    long long quota_u = 0;

    bool correct_quota = true;
    bool is_save = true;

    Lconfig conf(GLOBAL_SERVICE_LCONFIG_DIR); //config object
    if(conf.readConfig()!=0) std::cerr <<__FILE__<<" "<<__LINE__<< " Error working with config!"<< std::endl; 

    quota = conf.readLongInt(LSETTINGS_QUOTA_TOTAL);
//    std::cout <<"Syncconfig: Quota - "<<quota<< std::endl; 
    if(quota != -1)
        setQuoteTotal(quota);
    else
        correct_quota = false;

    quota_u = conf.readLongInt(LSETTINGS_QUOTA_USED);
//    std::cout <<"Syncconfig: Quota used - "<<quota_u<< std::endl; 
    if(quota_u != -1)
        setQuoteUsed(quota_u);
    else
        correct_quota = false;

    if (correct_quota)
        ui->progressBar->setToolTip(i18n("Used %1 Mb. of available %2 Mb.", quota_u / 1000 / 1000, quota / 1000 / 1000));

    string str =  (const char*)QVariant(m_login).toString().toLocal8Bit().data();
//    cout<<"Login - "<<str<<std::endl;
    if(conf.writeString(LSETTINGS_LOGIN, str) != 0) is_save = false;

    str =  (const char*)QVariant(m_password).toString().toLocal8Bit().data();
//    cout<<"Password - "<<str<<std::endl;
    if(conf.writeString(LSETTINGS_PASSWORD, str) != 0) is_save = false;

    if(conf.writeConfig()!=0)  
    {
       std::cerr <<__FILE__<<" "<<__LINE__<< " Error working with config!"<< std::endl; 
       is_save = false;
    }

    if(!is_save) {
        if (!nospace) {
            QMessageBox::critical(this, _SERVICE_NAME, i18n("Error saving settings"), QMessageBox::Ok);
            nospace = true;
        }
    } else
        nospace = false;
}

//! Функция установки доступного места на сервере
/**
  * \param bytes число свободных байт на сервере
  */
void PSyncConfig::setQuoteTotal(unsigned long long bytes)
{
    unsigned long long full = bytes / 1000 / 1000;

    ui->progressBar->setMaximum(full);
    ui->label_full->setText(QString::number(full) + i18n(" Mb."));
}

//! Функция установки занятого места на сервере
/**
  * \param bytes число занятых байт на сервере
  */
void PSyncConfig::setQuoteUsed(unsigned long long bytes)
{
    unsigned int value = bytes / 1000 / 1000;
    ui->progressBar->setValue(value);
    ui->label_half->setText(QString::number(value) + i18n(" Mb."));
}

void PSyncConfig::clearAuthData()
{
    m_login = m_password = "";
    saveSettings();
}

//! Функция инициализации полей формы
void PSyncConfig::initMargins()
{
    QString padding = "padding: 5px 10px 5px 10px;";

    ui->pushButton_save->setStyleSheet(padding);
    ui->pushButton_restore->setStyleSheet(padding);
    ui->pushButton_clear->setStyleSheet(padding);
    ui->pushButton_parameters->setStyleSheet(padding);
    ui->pushButton_new_save_point->setStyleSheet(padding);
    ui->pushButton_syncronize->setStyleSheet(padding);
    ui->pushButton_change_folder->setStyleSheet(padding);
}

//! Функция отправки уведомления об изменении настроек (из psyncconfig в psyncd)
void PSyncConfig::sendSettingsChangedNotify()
{    
    IPCPacket* pkt = IPCPacket::Create(SETTINGS_CHANGED, (char*)PSYNCCONFIG_IPC_NAME, 0,0);

    if (m_us->writeDatagram((const char*)pkt->getBuffer(), pkt->getSize(), PSYNCD_IPC_NAME) < 0)
    {
    	qDebug("IPC error with %s: type %lu",PSYNCD_IPC_NAME,pkt->getType());
        //QMessageBox::critical(this, QString("Sync Client"), i18n("The program error occurred. Information about the process and since the last synchronization may be displayed incorrectly"), QMessageBox::Ok);
    }

    if (pkt) delete pkt;
}

//! Слот - настройки изменились
void PSyncConfig::settingsChanged()
{
    saveSettings();
}

///
/// Syncronization checkbox
///

//! Функция отправки сигнала об изменении состояния синхронизации в модуль psyncnotify
void PSyncConfig::sendSyncronizeIs()
{
    qDebug("sendSyncronizeIs");

    if ((ui->checkBox_is_syncronized->checkState() == Qt::Checked))
        IPCMedia::send(PSYNCNOTIFY_IPC_NAME, PSYNCCONFIG_IPC_NAME, PSYNCNOTIFY_SYNC_ENABLED);
    else
        IPCMedia::send(PSYNCNOTIFY_IPC_NAME, PSYNCCONFIG_IPC_NAME, PSYNCNOTIFY_SYNC_DISABLED);
}

void PSyncConfig::refuseSaveFile()
{
    unsigned char answer[1] = { 0x30 };
    IPCMedia::send(PSYNCD_IPC_NAME, PSYNCCONFIG_IPC_NAME, PSYNCD_CONFLICT_SAVE_FILE, answer,sizeof(unsigned char));
}

void PSyncConfig::conflictFileToSave(const QString &filename, unsigned int timer)
{
    fileToSave = new QMessageBox(this);
    QAbstractButton *saveButton = fileToSave->addButton(i18n("Save"), QMessageBox::ActionRole);
    QAbstractButton *skipButton = fileToSave->addButton(i18n("Skip"), QMessageBox::ActionRole);
    fileToSave->setWindowIcon(QIcon(":synchronize.png").pixmap(128,128));
    fileToSave->setWindowTitle(_SERVICE_NAME);
    fileToSave->setText(i18n("File could not be save by present way"));
    fileToSave->setInformativeText(filename);
    fileToSave->setWindowModality(Qt::ApplicationModal);
    fileToSave->show();

    notify_timer = new QTimer(this);
    connect(notify_timer, SIGNAL(timeout()), fileToSave, SLOT(deleteLater()));
    notify_timer->start(timer*1000);

    connect(fileToSave, SIGNAL(destroyed()), this, SLOT(refuseSaveFile()));
    connect(saveButton, SIGNAL(clicked()), this, SLOT(callFileDialog()));
    connect(skipButton, SIGNAL(clicked()), fileToSave, SLOT(deleteLater()));
}

//! Функция отправки сигнала отображать/скрывать иконку в системном трее в модуль psyncnotify
void PSyncConfig::showTrayIconChanged()
{
    qDebug("showTrayIconChanged");

    if (ui->m_cb_showTrayIcon->checkState() == Qt::Checked)
    {
        IPCMedia::send(PSYNCNOTIFY_IPC_NAME, PSYNCCONFIG_IPC_NAME,PSYNCNOTIFY_SHOW_TRAY_ICON);
    }
    else
    {
        IPCMedia::send(PSYNCNOTIFY_IPC_NAME, PSYNCCONFIG_IPC_NAME,PSYNCNOTIFY_HIDE_TRAY_ICON);
    };

    Lconfig conf(GLOBAL_SERVICE_LCONFIG_DIR); //config object
    if(conf.readConfig()!=0)   std::cerr <<__FILE__<<" "<<__LINE__<< " Error working with config!"<< std::endl; 
    conf.writeInt(LSETTINGS_SHOW_TRAY, ui->m_cb_showTrayIcon->checkState());
    if(conf.writeConfig()!=0)  std::cerr <<__FILE__<<" "<<__LINE__<< " Error working with config!"<< std::endl; 
}

//! Функция отправки сигнала отображать/скрывать уведомления о начале и завершении синхронизации в модуль psyncnotify
void PSyncConfig::showStartEndNotifyChanged()
{
    qDebug("showStartEndNotifyChanged");

    if (ui->m_cb_showStartEndNotifications->checkState() == Qt::Checked)
    {
        IPCMedia::send(PSYNCNOTIFY_IPC_NAME, PSYNCCONFIG_IPC_NAME,PSYNCNOTIFY_SHOW_NOTIFY_START_END);
    }
    else
    {
        IPCMedia::send(PSYNCNOTIFY_IPC_NAME, PSYNCCONFIG_IPC_NAME,PSYNCNOTIFY_HIDE_NOTIFY_START_END);
    };

    Lconfig conf(GLOBAL_SERVICE_LCONFIG_DIR); //config object
    if(conf.readConfig()!=0)   std::cerr <<__FILE__<<" "<<__LINE__<< " Error working with config!"<< std::endl; 
    conf.writeInt(LSETTINGS_SHOW_NOTIFY_START_END, ui->m_cb_showStartEndNotifications->checkState());
    if(conf.writeConfig()!=0)  std::cerr <<__FILE__<<" "<<__LINE__<< " Error working with config!"<< std::endl; 
}

//! Функция отправки сигнала отображать/скрывать уведомления о синхронизации файлов в модуль psyncnotify
void PSyncConfig::showFileNotifyChanged()
{
    qDebug("showFileNotifyChanged");

    if (ui->m_cb_showFileNotifications->checkState() == Qt::Checked)
    {
        IPCMedia::send(PSYNCNOTIFY_IPC_NAME, PSYNCCONFIG_IPC_NAME,PSYNCNOTIFY_SHOW_NOTIFY_FILE);
    }
    else
    {
        IPCMedia::send(PSYNCNOTIFY_IPC_NAME, PSYNCCONFIG_IPC_NAME,PSYNCNOTIFY_HIDE_NOTIFY_FILE);
    };

    Lconfig conf(GLOBAL_SERVICE_LCONFIG_DIR); //config object
    if(conf.readConfig()!=0)   std::cerr <<__FILE__<<" "<<__LINE__<< " Error working with config!"<< std::endl; 
    conf.writeInt(LSETTINGS_SHOW_NOTIFY_FILE, ui->m_cb_showFileNotifications->checkState());
    if(conf.writeConfig()!=0)  std::cerr <<__FILE__<<" "<<__LINE__<< " Error working with config!"<< std::endl; 
}

//! Функция преобразования определенного поля графического элемента ComboBox в число секунд для интервала периодической синхронизации
/**
  * \param index номер поля в графическом элементе ComboBox
  * \return число секунд для интервала периодической синхронизации, 0 - если не выбрано ниодно из полей
  */
unsigned long PSyncConfig::fromIndexToSec(int index)
{
    if (index == 0)
        return 900;

    if (index == 1)
        return 1800;

    if (index == 2)
        return 3600;

    return 0;
}

//! Функция преобразования определенного числа секунд для интервала периодической синхронизации в номер поля графического элемента ComboBox
/**
  * \param Sec число секунддля интервала периодической синхронизации
  * \return номер поля в графическом элементе ComboBox, 4 - если значение времени не удовлетворяет ниодному из заданных промежутков времени
  */
int PSyncConfig::fromSecToIndex(unsigned long Sec)
{
    if (Sec == 900)
        return 0;

    if (Sec == 1800)
        return 1;

    if (Sec == 3600)
        return 2;

    return 4;
}

//! Слот, связанный с кнопкой добавления объектов в синхронизацию
void PSyncConfig::on_m_btn_addobject_clicked()
{
        /*QFileDialog dialog(this);
        //dialog.setFileMode(QFileDialog::ExistingFiles);
        dialog.setFileMode(QFileDialog::Directory);
        QStringList fileNames;
        if (dialog.exec()) fileNames = dialog.selectedFiles();*/

        QFileInfo fi_url;
        static QString path;
        fi_url.setFile(path);
        if (!fi_url.isWritable() || path.isEmpty())
            path = "file:///" + QString(getenv("HOME"));

        KUrl url(path);
        KFileDialog dialog(url,"*", this);
        dialog.setMode(KFile::Files | KFile::Directory);

        QStringList fileNames;
        QStringList fileAndDirNames;
        QStringList tmpList;
        QFileInfo   fi_checkdir;
        QStringList errorlist;
        QString     includelist;
        bool isDir= false;

        if (dialog.exec()) tmpList = dialog.selectedFiles();
        path = dialog.baseUrl().path();

        QFileInfo fi;
        foreach(QString file, tmpList) {
            tmpList.removeDuplicates();
            fi.setFile(file);
            if (!fi.exists()) {
                errorlist.push_back(file);
            }
            else {
                if (fi.filePath() == dialog.baseUrl().path()) {
                    fileAndDirNames.push_back(file);
                    isDir = true;
                    foreach(QString i_file, tmpList) {
                        fi_checkdir.setFile(i_file);
                        if (fi_checkdir.filePath() != dialog.baseUrl().path()) {
                            if (fi_checkdir.exists())
                                includelist += fi_checkdir.filePath() + "\n";
                            else
                                errorlist.push_back(fi_checkdir.filePath());
                        }
                    }
                    break;
                } else {
                    fileNames.push_back(file);
                }
            }
        }
        if (isDir) {
            fileNames.clear();
            fileNames = fileAndDirNames;
        }
        fileNames.replaceInStrings(QRegExp("(\\.){1,}$"), QString());
        fileNames.replaceInStrings(QRegExp("/{1,}$"), QString());

        fileNames.removeDuplicates();
        errorlist.removeDuplicates();
        if (!includelist.isEmpty())
            QMessageBox::warning(this, _SERVICE_NAME, i18n("Current directory was changed to ") + dialog.baseUrl().path()
                                                    + i18n("\nFollowing files are contained in here:\n")
                                                    + includelist, QMessageBox::Ok);
        if (!errorlist.isEmpty())
            QMessageBox::warning(this, _SERVICE_NAME, i18n("Following files does not exist:\n") + errorlist.join("\n"), QMessageBox::Ok);

	if(fileNames.empty())	return;
#ifdef SYNC_DB
        // TODO: new version of function with DB
        db_row db_obj;
        db_table table;
        db_expr expr;
        db_table::iterator cur;
        db_row::iterator r;
        bool isEqual;

        db->get_objects(expr, &table);
        foreach(QString file, fileNames) {
           qDebug() << "current file = " << file;
           isEqual = false;
        for(cur = table.begin(); cur < table.end(); ++cur) {
            db_row row = (db_row)*cur;
            for(r = row.begin(); r != row.end(); ++r)
                if ((QString::fromLocal8Bit((*r).first.c_str()) == "local_path")    // TODO:fromtoLocal8Bit
                        && (QString::fromLocal8Bit((*r).second.c_str()) == file))
                            isEqual = true;
        }
            if(!isEqual) {
                db_obj.insert(std::make_pair<string, string>("local_path", file.toLocal8Bit().data()));
                db->add_object(db_obj);
                db_obj.clear();
                ui->m_list_objects->addItem(file);
            }
        }
#else
        // TODO: debug information
/*        int rown = db->get_objects(expr, &table);
        std::cout << "Result: " << rown << "rowns" << std::endl;

        int i=1;
      //  db_table::iterator cur;
        for(cur=table.begin(); cur < table.end(); ++cur)
        {
            db_row row = (db_row)*cur;
        //    db_row::iterator r;
            std::cout << "\n Result " << i << ":\n" << std::endl;
            for(r = row.begin(); r != row.end(); ++r)
            {
                //expr.insert(*r);
                //db->delete_objects(expr);
                //if (QString::fromLocal8Bit((*r).first.c_str()) == "local_path")
                  //  ui->m_list_objects->addItem(QString::fromLocal8Bit((*r).second.c_str()));
                std::cout << "        " << (*r).first << "   -   " << (*r).second << std::endl;
            }
            ++i;
        }
*/
        // TODO: old version of function without DB
        QString s = fileNames.join("\n");
	qDebug("Adding files: %s",s.toLocal8Bit().data());

	IPCPacket * pkt = IPCPacket::Create(PSYNCD_ADD_TO_SYNC_COMMAND, (char*)PSYNCCONFIG_IPC_NAME, (unsigned char*)s.toLocal8Bit().data(), strlen((const char*)s.toLocal8Bit().data()));

	QString receiver_address(PSYNCD_IPC_NAME);


	if(m_us->writeDatagram((const char*)pkt->getBuffer(), pkt->getSize(), receiver_address.toLocal8Bit().data()) < 0)
	{
		qDebug("Error adding files: %s",s.toLocal8Bit().data());
	}

	if (pkt) delete pkt;
#endif

        on_tabWidget_currentChanged(ui->tabWidget->currentIndex());
}

//! слот, связанный с кнопкой удаления объектов из синхронизации
void PSyncConfig::on_m_btn_delobject_clicked()
{
        QList<QListWidgetItem *> fileNames;
        fileNames = ui->m_list_objects->selectedItems();

        if (fileNames.empty()) return;

        QListWidgetItem *item;
        QString s;
        QStringList files;

        foreach (item, fileNames) {
            if(!item->text().isEmpty()) {
                files.push_back(item->text());      // with DB
                s += item->text() + "\n";           // without DB
            }
        }

        int ret = QMessageBox::question(this, _SERVICE_NAME, i18n("Delete file %1 \nfrom syncronization?\n",files.join("\n")),QMessageBox::Ok| QMessageBox::Cancel);

        if(ret != QMessageBox::Ok) return;

        qDebug() << "Deleting files: " << files;
#ifdef SYNC_DB
        // TODO: new version of function with DB
        foreach(item, fileNames)
            ui->m_list_objects->takeItem(ui->m_list_objects->row(item));

        db_table::iterator cur;
        db_row::iterator r;
        db_table table;
        db_expr expr_mask;
        db_expr expr;

        db->get_objects(expr, &table);
        foreach(QString file, files) {
            qDebug() << "current file = " << file;
            for(cur = table.begin(); cur < table.end(); ++cur) {
                db_row row = (db_row)*cur;
                for(r = row.begin(); r != row.end(); ++r)
                    if ((QString::fromLocal8Bit((*r).first.c_str()) == "local_path")    // TODO:fromtoLocal8Bit
                            && (QString::fromLocal8Bit((*r).second.c_str()) == file)) {
                        expr_mask.insert(std::make_pair<string, string>("local_path", file.toLocal8Bit().data()));
                        expr.insert(std::make_pair<string, string>("flags", "1"));
                        db->update_objects(expr_mask, expr);
                        expr_mask.clear();
                        expr.clear();
                    }
            }
        }

#else
        // TODO: old version of unction without DB
	IPCPacket * pkt = IPCPacket::Create(PSYNCD_REMOVE_TO_SYNC_COMMAND, (char*)PSYNCCONFIG_IPC_NAME, (unsigned char*)s.toLocal8Bit().data(), strlen((const char*)s.toLocal8Bit().data()));

	QString receiver_address(PSYNCD_IPC_NAME);

	if(m_us->writeDatagram((const char*)pkt->getBuffer(), pkt->getSize(), receiver_address.toLocal8Bit().data()) < 0)
	{
		qDebug("Error Deleting file from psyncd");
	}

	if (pkt) delete pkt;
#endif
        on_tabWidget_currentChanged(ui->tabWidget->currentIndex());
}

//! Слот изменения номера позиции текущей вкладки, отправляет в модуль psyncd сигнал на получение всех синхронизируемых файлов
/**
  * \param index номер позиции
  */
void PSyncConfig::on_tabWidget_currentChanged(int index)
{
    if(index!=2) return;
    IPCPacket* pkt = IPCPacket::Create(PSYNCD_GET_ALL_FILES_COMMAND, (char*)PSYNCCONFIG_IPC_NAME,0,0);

    QString receiver_address(PSYNCD_IPC_NAME);

    if (m_us->writeDatagram((const char*)pkt->getBuffer(), pkt->getSize(), receiver_address.toLocal8Bit().data()) < 0)
    {
        qDebug("IPC error with %s: type %lu",receiver_address.toLocal8Bit().data(),pkt->getType());
        //QMessageBox::critical(this, QString("Sync Client"), i18n("The program error occurred. Information about the process and since the last synchronization may be displayed incorrectly"), QMessageBox::Ok);
    }

    if (pkt) delete pkt;
}
