//! \file psyncnotify.h Объявление класса приложения psyncnotify
#ifndef PSYNCNOTIFY_H
#define PSYNCNOTIFY_H

#include <QtGui>
#include <QFile>
#include <QSocketNotifier>
#include <QMessageBox>
#include <QMovie>
#include <QProgressBar>
#include <string>
#include <map>
#include <kprocess.h>
#include <knotification.h>
#include <klocale.h>
#include <kglobal.h>
#include <kapplication.h>
#include <kmenu.h>
#include <kstatusnotifieritem.h>
#include <kactioncollection.h>  // TODO:
#include <iostream>             // TODO:

//#include <kstandardaction.h>  // TODO: changed standart quit
#include "ipc_interface.h"
#include "ipc_interface_local.h"
#include "ipcpacket.h"
#include "unixsocket.h"
#include "progressnotifyjob.h"

//! Класс, описывающий индикатор загрузки файлов на сервер
class ProgressNotifier
{
public:
	ProgressNotifier();
	~ProgressNotifier();
	void setText(QString text);
	void setPercent(int value);
        QWidgetAction * m_text_action;      //!< Имя файла, отображаемое в меню в момент его загрузки на сервер
        QWidgetAction * m_progress_action;  //!< Индикатор загрузки одного файла, отображаемый в меню
        QLabel*		m_label;            //!< Текстовая метка с именем загружаемого файла
        QProgressBar*	m_progress;         //!< Текущий индикатор процесса загрузки файла на сервер
};

//! Класс, описывающий встраивание индикатора в меню
class ProgressNotifierArea
{
public:
	ProgressNotifierArea();
	~ProgressNotifierArea();

        bool attachToMenu(KMenu* menu, QAction * before); // TODO: QMenu
	void detach();

	bool add(QString name);
	void setText(QString name, QString text);
	void setPercent(QString name, int value);
	void remove(QString name);
	bool exist(QString name);
protected:

        QMap<QString,ProgressNotifier*> m_notifiers;    //!< Ассоциативный массив, включающий пару: имя файла и индикатор загрузки
        KMenu * m_menu;                                 //!< Меню с индикатором загрузки файлов
        QAction*		m_sep1;                 //!< Действие добавляемое в меню
        QAction*		m_sep2;                 //!< Не используется
        QAction*		m_before;               //!< Действие в меню, перед которым будет добавлено имя файла и индикатор его загрузки
};

//! Главный класс приложения
class PSyncNotify : public KStatusNotifierItem
{
    Q_OBJECT

public:
    PSyncNotify(KApplication *app, QWidget *parent = 0);
    ~PSyncNotify();

private slots:
    void receiveNotification();
    void iconActivated(QSystemTrayIcon::ActivationReason reason);
    void managedNotifyActivated(const QString &title, const QString& filename, int mode, unsigned int timer, int option);
    void generalNotify(const QString &title, const QString &text, const QPixmap &pixmap = QPixmap(), QWidget *widget = 0L);
    void sendConflictFirst();       //
    void sendConflictSecond();      // TODO: notify menu conflicts
    void sendConflictThird();       //
    void updateMenuPosAndSize();
    void syncronizeStartStop();
    void openSettings();
    void closeApp();
    void updateIcon();
    void Notify_Close();
    void updateLastSyncTimeLabel();
    //void slotQuit(); // TODO: changed standart quit
    void activate(bool active, const QPoint &pos);
    //void activate(const QPoint &pos);
    void singleModeSaveFile();
    void refuseSaveFile();
    void recursiveSave();

protected:
    bool eventFilter(QObject *o, QEvent *e);    // TODO: try catch mouse right click

private:
    void callFileDialog(int opt);
    void createActions();
    void createTrayIcon();
    QLabel *createLabel(QWidget *widget, const QString &text, Qt::AlignmentFlag aligment, int margin);
    QFont createFont(const QString &name, int size, bool is_bold, bool is_italic);

    inline void initSocket();
    inline void readSettings();

    void setLastSyncTimeLabel(QString str);
    void setToolTipStopLoadFile();
    void restoreAfterStopSync();
    void reportSyncStart();
    void reportSyncStop();

private:
    QWidget *referenceWidget;           //!< Объект QWidget
    QAction *lastsynctimeAction;  //!< Поле в меню, содержащее время последней синхронизации
    QAction *syncrAction;               //!< Поле в меню, содержащее действие синхронизации
    QAction *opensettingsAction;        //!< Поле в меню, содержащее действие для открытия приложения настроек
    //std::map<QString,KNotification*> m_notificationMap;

    std::map<QString,ProgressNotifyJob*> m_notificationMap; // TODO: no where used! May be del?

    ProgressNotifierArea m_na;          //!< Объект класса, описывающего встраивание индикатора в меню

    QSystemTrayIcon *trayIcon;          //!< Не используется
    KMenu *trayIconMenu;                //!< Меню приложения
    QIcon *m_icon;                      //!< Иконка приложения
    QStringList collectDirToSave;
    QMap<QString, QString> recursiveMap;
    QString tmp_string;
    QMessageBox *fileToSave;

    UnixSocket *m_us;                               //!< Объект класса UnixSocket из библиотеки libpsync
    QSocketNotifier *m_sn;                          //!< Объект для поддержки мониторинга активности файлового дескриптора

    QTimer       *notify_timer;

    bool         isRecursiveSaving;
    bool 	 m_sync_on;                         //!< Синхронизация включена/выключена
    bool 	 m_sync_is_running;                 //!< Синхронизация запущена/остановлена
    int 	 m_show_tray_icon;                  //!< Показывать/скрывать иконку в системном трее
    int          m_show_start_end_notifications;    //!< Показвать/скрывать уведомления о начале/завершении синхронизации
    int          m_show_file_notifications;         //!< Показвать/скрывать уведомления о синхронизации файлов
    unsigned int m_timer;                           //!< Время отображения уведомления о разрешении коллизий

    KApplication *m_app;                            //!< Объект для управления KDE приложением
    QMovie *m_gif;                                  //!< Анимационная иконка
};

#endif // PSYNCNOTIFY_H
