//! \file psyncnotify.cpp Реализация классов приложения
#include "../libpsync/include/sync_global.h"
#include "../psyncconfig/include/config.h"
#include "../syncd/include/ipc_interface.h"
#include "../psyncconfig/include/ipc_interface.h"
#include "ipcmedia.h"
#include "psyncnotify.h"
#include "ipc_interface.h"
//#include "progressnotifyjob.h"

#include "../liblcfg/include/lconfig.h"

#include <kiconloader.h>
#include <kicon.h>

//#define _PROGRAMM_NAME 			i18n(GLOBAL_SERVICE_PRODUCT_NAME)
#define _SERVICE_NAME 			i18n(GLOBAL_SERVICE_PRODUCT_NAME)                           //!< Общее название продукта
#define _PROGRAMM_NAME 			i18n("%1 client", QString(GLOBAL_SERVICE_PRODUCT_NAME))     //!< Название приложения
#define _NOTIFICATION_CAPTION	i18n(GLOBAL_SERVICE_PRODUCT_NAME)                                   //!< Заголовок уведомлений

//! Конструктор класса индикатора загрузки файлов на сервер
ProgressNotifier::ProgressNotifier()
{
	m_label = NULL;
	m_label = new QLabel("");
	m_label->setStyleSheet(QString(" QLabel { \
		     border: 0px solid grey;\
			 margin: 3px 6px 0px 6px;\
			 text-align: center;\
			background-color: transparent;\
		 }"));

	m_progress = NULL;
	m_progress = new QProgressBar();
	m_progress->setMinimum(0);
	m_progress->setMaximum(100);
	m_progress->setValue(0);
	m_progress->setStyleSheet(QString(" QProgressBar { \
	     border-radius: 5px;\
		 height: 20px;\
		 border: 0px solid grey;\
		 margin: 0px 6px 3px 6px;\
		 text-align: center;\
		background-color: transparent;\
	 }\
	 QProgressBar::chunk {\
	     background-color: #76b3e8;\
	     width: 20px;\
	 }"));

        m_text_action = NULL;
        m_text_action = new QWidgetAction(NULL);
        m_text_action->setDefaultWidget(m_label);

	m_progress_action = NULL;
        m_progress_action = new QWidgetAction(NULL);
        m_progress_action->setDefaultWidget(m_progress);
}

//! Деструктор класса индикатора загрузки файлов на сервер
ProgressNotifier::~ProgressNotifier()
{
	if(m_progress_action) delete m_progress_action;
	if(m_text_action) delete m_text_action;

	m_text_action = NULL;
	m_label	= NULL;
	m_progress = NULL;
	m_progress_action = NULL;
}

//! Функция установки текста на индикатор и всплывающую подсказку
/**
  * \param text утанавливаемый текст
  */
void ProgressNotifier::setText(QString text)
{
	m_label->setToolTip(text);
	m_progress->setToolTip(text);
	if(text.length()>60) text=text.left(60)+"...";
	m_label->setText(text);

}

//! Функция установки текущего значения индикатора
/**
  * \param value текущее значение индикатора
  */
void ProgressNotifier::setPercent(int value)
{
	m_progress->setValue(value);
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////

//! Конструктор класса встраивания индикатора в меню
ProgressNotifierArea::ProgressNotifierArea()
{
	m_menu=NULL;
	m_before = NULL;
	m_sep1=NULL;
	m_sep2=NULL;
}

//! Деструктор класса
/**
  * Убирает индикатор из меню
  */
ProgressNotifierArea::~ProgressNotifierArea()
{
	detach();
}

//! Определение возможности присоединения индикатора к меню
/**
  * \param menu меню главного приложения без индикатора
  * \param before строка в меню перед которой будет добавлен индикатор
  * \return true - можно присоединять к меню индикатор, false - меню с индикатором уже существует, присоединение не возможно
  */
bool ProgressNotifierArea::attachToMenu(KMenu* menu, QAction * before)
{
	if(m_menu) return false;
	m_menu = menu;
	m_before = before;
	return true;
}

//! Функция удаления индикатора из меню
void ProgressNotifierArea::detach()
{
	if(!m_menu) return;
	m_menu->removeAction(m_sep1);

	for (QMap<QString,ProgressNotifier*>::iterator i = m_notifiers.begin(); i != m_notifiers.end(); ++i)
	{
	     m_menu->removeAction(m_notifiers[i.key()]->m_text_action);
	     m_menu->removeAction(m_notifiers[i.key()]->m_progress_action);
	}
	m_notifiers.clear();
}

//! Определение состояния добавления нового файла
/**
  * Для каждого нового файла поступившего в синхронизацию, создается свой индикатор,
  * определяющий стпень загруженности файла на сервер
  * \param name имя файла
  * \return true - если файл уже находится в состоянии загрузки, false - если меню с индикатором не было создано
  */
bool ProgressNotifierArea::add(QString name)
{
	if(!m_menu) return false;
	if(m_notifiers[name]) return true;
	m_notifiers[name]= new ProgressNotifier();

	if(!m_sep1)
	{
		m_sep1 = m_menu->insertSeparator(m_before);
	}

	m_menu->insertAction(m_sep1,m_notifiers[name]->m_text_action);
	m_menu->insertAction(m_sep1,m_notifiers[name]->m_progress_action);
	return true;
}

//! Функция установки текста для текущего файла
/**
  * \param name имя текущего файла
  * \param text устанавливаемый текст
  */
void ProgressNotifierArea::setText(QString name, QString text)
{
	if(!m_notifiers[name]) return;
	m_notifiers[name]->setText(text);
}

//! Функция установки значения индикатора для текущего файла
/**
  * \param name имя текущего файла
  * \param value устанавливаемое значение индикатора
  */
void ProgressNotifierArea::setPercent(QString name, int value)
{
	if(!m_notifiers[name]) return;
	m_notifiers[name]->setPercent(value);
}

//! Функция освобождения памяти, занимаемой текущим файлом-объектом
/**
  * \param name имя текущего файла
  */
void ProgressNotifierArea::remove(QString name)
{
	if(!m_menu) return;
	if(!m_notifiers[name]) return;

	m_menu->removeAction(m_notifiers[name]->m_text_action);
	m_menu->removeAction(m_notifiers[name]->m_progress_action);
	delete m_notifiers[name];
	m_notifiers.remove(name);
}

//! Функция проверки существования текущего файла
/**
  * \param name имя текущего файла
  * \return true - файл существует, false - файл не существует
  */
bool ProgressNotifierArea::exist(QString name)
{
	if(!m_notifiers[name]) return false;
	return true;
}


/////////////////////////////////////////////////////////////////////////////////////////////////////////


//! Конструктор главного класса приложения
/**
  * \param app объект управления KDE приложением
  * \param parent объект-родитель
  */
PSyncNotify::PSyncNotify(KApplication *app,QWidget *parent)
    : KStatusNotifierItem(0)
    , referenceWidget(parent)
{
    m_app = app;
    isRecursiveSaving               = false;
    m_sync_is_running               = false;
    m_show_tray_icon                = true;
    m_show_start_end_notifications  = true;
    m_show_file_notifications       = true;
    m_sync_on                       = false;
    m_icon                          = new QIcon(QString(":/red_status.png"));
    m_gif                           = new QMovie(":animation.gif");

    readSettings();

    setToolTipTitle("MandrivaSync");
    setToolTipIconByPixmap(QIcon(":synchronize.png").pixmap(32,32));
    setTitle("MandrivaSync");
    setStatus(Active);
    setCategory(ApplicationStatus);
    setStandardActionsEnabled(false); // TODO: del standart 'quit'

    createActions();
    initSocket();
    createTrayIcon();

    setAssociatedWidget(trayIconMenu);
    contextMenu()->clear();

    // Setup GUI state according to readed config
    updateLastSyncTimeLabel();

    if(m_sync_on)
    {
        syncrAction->setEnabled(true);
        setIconByPixmap(QIcon(":synchronize.png").pixmap(24,24));
    } else {
        syncrAction->setEnabled(false);
        setIconByPixmap(QIcon(":red_status.png").pixmap(24,24));
    }

    connect(this, SIGNAL(activateRequested(bool,QPoint)), this, SLOT(activate(bool,QPoint)));
    connect(m_gif, SIGNAL(frameChanged(int)), this, SLOT(updateIcon()));

    /*const char *quitName = KStandardAction::name( KStandardAction::Quit );
    QAction *quit = ac->action( quitName );

    quit->disconnect( SIGNAL(triggered(bool)), this, SLOT(maybeQuit()) );
    connect( quit, SIGNAL(activated()), SLOT(slotQuit()) );
    */ // TODO: changed standart quit

    qApp->installEventFilter(this); // TODO:

    IPCMedia::send(PSYNCD_IPC_NAME, PSYNCNOTIFY_IPC_NAME, PSYNCD_GET_SYNC_STATUS);
}

//! Деструктор класса
/**
  * Закрывает сокет и удаляет объект класса UnixSocket
  */
PSyncNotify::~PSyncNotify()
{
    m_us->finish();
    delete m_us;

/*    for(map<string,KNotification*>::iterator i=m_notificationMap.begin();i!=m_notificationMap.end();++i){
		i->second->deref();
		i->second->close();
		i->second=NULL;
	}
    m_notificationMap.clear();*/
}

bool PSyncNotify::eventFilter(QObject *o, QEvent *e)
{
    /*if (e->type() == QEvent::MouseButtonPress) {
        std::cout << "mouse are press" << std::endl;
    }
    if (e->type() == QEvent::MouseButtonRelease) {
        std::cout << "mouse are release" << std::endl;
    }
    if (e->type() == QEvent::MetaCall) {
        std::cout << "context menu" << std::endl;
    }*/
    //std::cout << "type = " << e->type() << std::endl;
    return QObject::eventFilter(o, e);
}

//! Функция создания действий для меню
void PSyncNotify::createActions()
{
    KIconLoader loader;
    QIcon icon_settings;
    icon_settings = loader.loadIconSet("configure", KIconLoader::Small);

    lastsynctimeAction = new QAction(this);
    lastsynctimeAction->setText(i18n("The last synchronization: "));
    lastsynctimeAction->setEnabled(false);

    syncrAction = new QAction(i18n("Syncronize"), this);
    syncrAction->setEnabled(false);
    connect(syncrAction, SIGNAL(triggered()), this, SLOT(syncronizeStartStop()) );

    opensettingsAction = new QAction(icon_settings, i18n("Settings..."), this);
    connect(opensettingsAction, SIGNAL(triggered()), this, SLOT(openSettings()));
}

//! Функция закрытия приложения
void PSyncNotify::closeApp()
{
    m_app->setQuitOnLastWindowClosed(true);
    //close();
}

//! Функция создания иконки в системном трее
void PSyncNotify::createTrayIcon()
{
    trayIconMenu = new KMenu(referenceWidget);
    trayIconMenu->addTitle(QIcon(":synchronize.png").pixmap(24,24), _SERVICE_NAME, lastsynctimeAction);
    trayIconMenu->addSeparator();
    trayIconMenu->addAction(lastsynctimeAction);
    trayIconMenu->addAction(syncrAction);
    trayIconMenu->addAction(opensettingsAction);

    m_na.attachToMenu(trayIconMenu,lastsynctimeAction);

    if(m_show_tray_icon) {
        QIcon iconEnabled  = QIcon( *m_icon );
        if (iconEnabled.isNull()) {
            QMessageBox::warning(referenceWidget, _PROGRAMM_NAME, i18n("Cannot load system tray icon." ),QMessageBox::Ok);
        }else {
            setIconByPixmap(iconEnabled.pixmap(24,24));
        }
    }
}

//! Функция-обертка класса QLabel
/**
  * Создает метку с отформатированным текстом
  * \param widget объект-родитель
  * \param text текст, отображаемый в метке
  * \param aligment выравнивание
  * \param margin поле
  * \return указатель на объект типа QLabel
  */
QLabel* PSyncNotify::createLabel(QWidget *widget, const QString &text, Qt::AlignmentFlag aligment, int margin)
{
    QLabel *label = new QLabel(widget);
    label->setText(text);
    label->setAlignment(aligment);
    label->setMargin(margin);
    return label;
}

//! Функция-обертка класса QFont
/**
  * Создает шаблон шрифта для текста
  * \param name название шрифта
  * \param size размер шрифта
  * \param is_bold парамерт шрифта: true - жирный, false - обычный
  * \param is_italic паарметр стиля шрифта: true - курсив, false - обычный
  * \return объект типа QFont
  */
QFont PSyncNotify::createFont(const QString &name, int size, bool is_bold, bool is_italic)
{
    QFont font(name, size);
    font.setBold(is_bold);
    font.setItalic(is_italic);

    return font;
}

void PSyncNotify::generalNotify(const QString &title, const QString &text, const QPixmap &pixmap, QWidget *widget)
{
    KNotification *notification = new KNotification("conflict", widget, KNotification::CloseOnTimeout);
    notification->setPixmap(pixmap);
    notification->setTitle(title);
    notification->setText(text);

    notification->sendEvent();
}

/* TODO: notification conflict*/
//! Функция создания/обработки уведомления о разрешении конфликтов
/**
  * \param title заголовок нотификатора
  * \param filename имя файла, с которым произошел конфликт (отображается в главном поле уведомления)
  * \param mode режим конфликта
  * \param timer время отображения уведомления (по истечению этого промежутка времени уведомление закроется)
  */
void PSyncNotify::managedNotifyActivated(const QString &title, const QString& filename, int mode, unsigned int timer, int option)
{
    if (!option) {
        KNotification *notification = new KNotification("conflict", referenceWidget, KNotification::Persistent);
        QStringList conflictlist;
            switch(mode)
            {
                case 0:
                    conflictlist << i18n("To server") << i18n("To client") << i18n("Default");
                    break;
                case 1:
                    conflictlist << i18n("To ignore") << i18n("Delete from the base") << i18n("Default (case: 2)");
                    break;
            }

        notification->setPixmap(QIcon(":synchronize.png").pixmap(32,32));
        notification->setTitle(title);
        notification->setText(filename);
        notification->setActions(conflictlist);

        notification->sendEvent();
        qDebug("timer = %d", timer); // TODO: timer

        QTimer::singleShot(timer*1000, notification, SLOT(close()));

        /*notify_timer = new QTimer(this);
        connect(notify_timer, SIGNAL(timeout()), notification, SLOT(close()));
        notify_timer->start(timer*1000);*/
        connect(notification, SIGNAL(action1Activated()), this, SLOT(sendConflictFirst()));
        connect(notification, SIGNAL(action2Activated()), this, SLOT(sendConflictSecond()));
        connect(notification, SIGNAL(action3Activated()), this, SLOT(sendConflictThird()));
        connect(notification, SIGNAL(closed()), this, SLOT(sendConflictThird()));
        connect(notification, SIGNAL(ignored()), this, SLOT(sendConflictThird()));
    } else {
        fileToSave = new QMessageBox(referenceWidget);
        QAbstractButton *saveButton = fileToSave->addButton(i18n("Save"), QMessageBox::ActionRole);
        QAbstractButton *skipButton = fileToSave->addButton(i18n("Skip"), QMessageBox::ActionRole);
        fileToSave->setWindowIcon(QIcon(":synchronize.png").pixmap(128,128));
        fileToSave->setWindowTitle(_SERVICE_NAME);
        fileToSave->setText(title);
        fileToSave->setInformativeText(filename);
        fileToSave->setWindowModality(Qt::ApplicationModal);
        fileToSave->show();

        notify_timer = new QTimer(this);
        connect(notify_timer, SIGNAL(timeout()), fileToSave, SLOT(deleteLater()));
        notify_timer->start(timer*1000);

        connect(fileToSave, SIGNAL(destroyed()), this, SLOT(refuseSaveFile()));
        connect(saveButton, SIGNAL(clicked()), this, SLOT(singleModeSaveFile()));
        connect(skipButton, SIGNAL(clicked()), fileToSave, SLOT(deleteLater()));
    }


    // Default way. If notification has disappeared or was ignored, then it choosed a default way.

    // Show notification
}

//! Не используется
void PSyncNotify::iconActivated(QSystemTrayIcon::ActivationReason reason)
{
    Q_UNUSED(reason);
    /*
        int x_pos;
	trayIconMenu->adjustSize();
	QRect d = QApplication::desktop()->screenGeometry();
	//qDebug("Desktop x=%d,y=%d,h=%d,w=%d",d.x(),d.y(),d.height(),d.width());
	QRect m = trayIconMenu->geometry();
	//qDebug("Menu x=%d,y=%d,h=%d,w=%d",r.x(),r.y(),r.height(),r.width());
	QRect i = trayIcon->geometry();
	//qDebug("Icon x=%d,y=%d,h=%d,w=%d",i.x(),i.y(),i.height(),i.width());
	x_pos = d.width()-m.width();
	if(x_pos>i.x()) x_pos=i.x();
	trayIconMenu->move(x_pos,i.y()-m.height());
	trayIconMenu->show();
     */
}
/*
void PSyncNotify::activate(const QPoint &pos)
{
    if (trayIconMenu->isVisible())
        trayIconMenu->hide();
    else
        trayIconMenu->show();

    trayIconMenu->adjustSize();
    int h = trayIconMenu->height();
    int x = pos.x();                //TODO: int x = pos.x() - trayIconMenu->width()/2;
    int y = pos.y() - h;

    if(y < 0) {
        y = pos.y();
    }

    trayIconMenu->move(x, y);

    const QDesktopWidget *vdesktop = QApplication::desktop();
    const QRect &vScreenSize = vdesktop->screenGeometry(trayIconMenu);

    if ((x + trayIconMenu->width()) > (vScreenSize.width() + vScreenSize.x())) {
        trayIconMenu->move(vScreenSize.width() + vScreenSize.x() - trayIconMenu->width() - 1, y);
    }
    else if (x < vScreenSize.x()) {
        trayIconMenu->move(vScreenSize.x(), y);
    }
}
*/

//! Функция установки позиции отображения меню
/**
  * \param active не используется
  * \param pos считанные координаты после клика мышкой по меню
  */
void PSyncNotify::activate(bool active, const QPoint &pos)
{
    Q_UNUSED(active);
    trayIconMenu->adjustSize();
    int h = trayIconMenu->height();
    int x = pos.x();                //TODO: int x = pos.x() - trayIconMenu->width()/2;
    int y = pos.y() - h;

    if(y < 0) {
        y = pos.y();
    }
    trayIconMenu->move(x, y);
    //qDebug() << "1 moving to" << trayIconMenu->size() << x << y;

    const QDesktopWidget *vdesktop = QApplication::desktop();
    const QRect &vScreenSize = vdesktop->screenGeometry(trayIconMenu);

    if ((x + trayIconMenu->width()) > (vScreenSize.width() + vScreenSize.x())) {
        trayIconMenu->move(vScreenSize.width() + vScreenSize.x() - trayIconMenu->width() - 1, y);
    }
    else if (x < vScreenSize.x()) {
        trayIconMenu->move(vScreenSize.x(), y);
    }
}

//! Не используется
void PSyncNotify::updateMenuPosAndSize()
{
/*        int x_pos;
        int y_pos;
	trayIconMenu->adjustSize();
	QRect d = QApplication::desktop()->screenGeometry();
        qDebug("Desktop x=%d,y=%d,h=%d,w=%d",d.x(),d.y(),d.height(),d.width());
	QRect m = trayIconMenu->geometry();
        qDebug("Menu x=%d,y=%d,h=%d,w=%d",m.x(),m.y(),m.height(),m.width());
        QRect i = contextMenu()->geometry();
        qDebug("Icon x=%d,y=%d,h=%d,w=%d",i.x(),i.y(),i.height(),i.width());
	x_pos = d.width()-m.width();
        y_pos = d.height()-m.height();
        qDebug("Menu move x_pos=%d,y_pos=%d",x_pos,y_pos);
        //if(x_pos>i.x()) x_pos=i.x();
        trayIconMenu->move(y_pos,x_pos);
        */
}

//! Не используется
void PSyncNotify::Notify_Close()
{
/*	string name;
	KNotification* src= qobject_cast<KNotification*>(sender());
	for(map<QString,KNotification*>::iterator i=m_notificationMap.begin();i!=m_notificationMap.end();++i)
		if(i->second == src){
			i->second=NULL;
		}
*/
}

//! Обработка сообщений от модулей psyncd и psyncconfig
void PSyncNotify::receiveNotification()
{
    QString msg;
    IPCPacket *m_pkt = NULL;
    int total_bytes = m_us->bytesAvailable();
    bool is_receiving_file =false;
    if (m_pkt)
        return;

    m_pkt = new IPCPacket();
    m_us->readDatagram((char*)m_pkt->readBuffer(total_bytes), total_bytes);
    QString filename;
    QString per;    // TODO: percent for tooltip
    int persent;
    int mode_conflict = 0;
    bool isContainsDirToSave = false;   // TODO: recursive saving
    QMap<QString, QString>::const_iterator i;

    // TODO: Use 'if (!m_pkt->IsValid()) return;' Luke!!
    if (m_pkt->IsValid())
    {
    	//qDebug("IPCPacket received: %lu datalen: %lu",m_pkt->getType(),m_pkt->getDataLen());
        switch (m_pkt->getType())
        {
            case PSYNCNOTIFY_NOTIFICATION_MESSAGE:
            generalNotify(_NOTIFICATION_CAPTION, QString::fromLocal8Bit((const char*)m_pkt->getData(), m_pkt->getDataLen()), QIcon(":synchronize.png").pixmap(32,32), referenceWidget);
                break;
	    case PSYNCNOTIFY_FILERECV_PROGRESS:
	    	is_receiving_file=true;
	    case PSYNCNOTIFY_FILESEND_PROGRESS:
			if(m_pkt->getDataLen()<=sizeof(int)) break;
			persent=*((int*)(m_pkt->getData()));
			filename = QString::fromLocal8Bit(QByteArray((const char*)m_pkt->getData()+sizeof(int),m_pkt->getDataLen()-sizeof(int)));
			//qDebug("PSYNCNOTIFY_FILE*_PROGRESS: %s, %d %",filename.toLocal8Bit().data(),persent);

			if(!m_na.exist(filename)) m_na.add(filename);

                        if(is_receiving_file)
                            m_na.setText(filename,i18n("Updating \"%1\"",filename));
                        else
                            m_na.setText(filename,i18n("Commiting \"%1\"",filename));

			m_na.setPercent(filename,persent);
                        setToolTipSubTitle(filename + "\n" + i18n("Progress: ") + per.setNum(persent) + "%" ); // TODO: tooltip percent
                        //updateMenuPosAndSize();
			break;
			
	    case PSYNCNOTIFY_HTTP_ERROR:
			filename=QString::fromLocal8Bit(QByteArray((const char*)m_pkt->getData(),m_pkt->getDataLen()));
                        if (filename.isEmpty())
                            msg = i18n("Server interaction error");
                        else
                            msg = i18n("Server interaction error (%1)!",filename);
                        generalNotify(_NOTIFICATION_CAPTION, msg, QIcon(":synchronize.png").pixmap(32,32), referenceWidget);
                        restoreAfterStopSync();
			break;
	    case PSYNCNOTIFY_FILERECV_ERROR:
   	    	is_receiving_file=true;
	    case PSYNCNOTIFY_FILESEND_ERROR:
			filename=QString::fromLocal8Bit(QByteArray((const char*)m_pkt->getData(),m_pkt->getDataLen()));
			if(is_receiving_file){
				msg = i18n("Error updating \"%1\"!",filename); }
			else{
				msg = i18n("Error commiting \"%1\"!",filename); }

                        generalNotify(_NOTIFICATION_CAPTION, msg, QIcon(":synchronize.png").pixmap(32,32), referenceWidget);
			m_na.remove(filename);
                        //updateMenuPosAndSize();
			break;
	    case PSYNCNOTIFY_FILERECV_NOSPACE:
			filename=QString::fromLocal8Bit(QByteArray((const char*)m_pkt->getData(),m_pkt->getDataLen()));
                        generalNotify(_NOTIFICATION_CAPTION, i18n("Error updating \"%1\": not enough disk space!",filename), QIcon(":synchronize.png").pixmap(32,32), referenceWidget);
			m_na.remove(filename);
                        //updateMenuPosAndSize();
			break;
	    case PSYNCNOTIFY_FILERECV_COMPLETED:
	    	is_receiving_file=true;
	    case PSYNCNOTIFY_FILESEND_COMPLETED:
                        filename=QString::fromLocal8Bit(QByteArray((const char*)m_pkt->getData(),m_pkt->getDataLen()));
                        if(is_receiving_file)
                            msg =  i18n("File \"%1\" successfully updated",filename);
                        else
                            msg =  i18n("File \"%1\" successfully commited!",filename);

                        if (m_show_file_notifications) {
                            generalNotify(_NOTIFICATION_CAPTION, msg, QIcon(":synchronize.png").pixmap(32,32), referenceWidget);
                        }
			m_na.remove(filename);
                        //updateMenuPosAndSize();
			break;
	    case PSYNCNOTIFY_SYNC_START:
            case PSYNCD_ANS_SYNC_STATUS_ONLINE:
			m_sync_is_running = true;
			syncrAction->setText(i18n("Stop synchronization"));
			syncrAction->setEnabled(true);
                        //((QLabel*)lastsynctimeAction->defaultWidget())->setVisible(false);
                        lastsynctimeAction->setVisible(false);
                        //updateMenuPosAndSize();
			m_gif->start();
                        if (m_show_start_end_notifications) {
                            reportSyncStart();
                        }
			break;
	    case PSYNCNOTIFY_SYNC_STOP:
                        restoreAfterStopSync();
                        //((QLabel*)lastsynctimeAction->defaultWidget())->setVisible(true);
                        //updateMenuPosAndSize();
                        if (m_show_start_end_notifications) {
                            reportSyncStop();
                        }
			// update last sync
			break;
		case PSYNCNOTIFY_USER_STOP:
                        restoreAfterStopSync();
                        if (m_show_start_end_notifications) {
                                generalNotify(_NOTIFICATION_CAPTION, i18n("Synchronization stopped!"), QIcon(":synchronize.png").pixmap(32,32), referenceWidget);
                        }
			break;
                case PSYNCNOTIFY_SYNC_ENABLED:
			syncrAction->setEnabled(true);
                        setIconByPixmap(QIcon(":synchronize.png").pixmap(24,24));
			break;
                case PSYNCNOTIFY_SYNC_DISABLED:
			syncrAction->setEnabled(false);
                        setIconByPixmap(QIcon(":red_status.png").pixmap(24,24));
			break;
		case PSYNCNOTIFY_SHOW_TRAY_ICON:
                        //trayIcon->show();     // TODO:
                        setStatus(Active);
                        m_show_tray_icon = true;
			break;
		case PSYNCNOTIFY_HIDE_TRAY_ICON:
                        //trayIcon->hide();     //TODO:
                        setStatus(Passive);
                        m_show_tray_icon = false;
			break;
                case PSYNCD_ANS_SYNC_STATUS_OFFLINE:
                        break;
                case PSYNCNOTIFY_SHOW_NOTIFY_START_END:
                        m_show_start_end_notifications = true;
                        break;
                case PSYNCNOTIFY_HIDE_NOTIFY_START_END:
                        m_show_start_end_notifications = false;
                        break;
                case PSYNCNOTIFY_SHOW_NOTIFY_FILE:
                        m_show_file_notifications = true;
                        break;
                case PSYNCNOTIFY_HIDE_NOTIFY_FILE:
                        m_show_file_notifications = false;
                        break;
                case PSYNCNOTIFY_FILEDIALOG:
                        filename=QString::fromLocal8Bit(QByteArray((const char*)m_pkt->getData(),m_pkt->getDataLen()));
                        generalNotify(_NOTIFICATION_CAPTION, i18n("For the file ") +
                                             filename + i18n("\ndirectory not been selected for saving. \n") +
                                             i18n("This file not be loaded from the server"), QIcon(":synchronize.png").pixmap(32,32), referenceWidget);
                        // TODO: notify
                        break;
                case PSYNCNOTIFY_CONFLICT_SAVE_FILE:
                        readSettings();
                        filename=QString::fromLocal8Bit(QByteArray((const char*)m_pkt->getData(),m_pkt->getDataLen()));
                        qDebug() << "filename(psyncnotify) = " << filename;
                        tmp_string = filename.left(filename.lastIndexOf(QRegExp("/")));
                        if(isRecursiveSaving) {
                            for(i = recursiveMap.constBegin(); i != recursiveMap.constEnd(); ++i) {
                                if (filename.contains(i.key(), Qt::CaseInsensitive)) {
                                    isContainsDirToSave = true;
                                    IPCMedia::send(PSYNCD_IPC_NAME, PSYNCNOTIFY_IPC_NAME, PSYNCD_CONFLICT_SAVE_FILE, (unsigned char*)(i.value().toLocal8Bit().data()), i.value().size());
                                    break;
                                }
                            }
                            if (!isContainsDirToSave)
                                managedNotifyActivated(i18n("File could not be save by present way"), filename, 0, m_timer, 1);
                        }
                        else {
                            if(!QFile::exists(PSYNCCONFIG_IPC_NAME)) {
                              /*  IPCMedia::send(PSYNCCONFIG_IPC_NAME, PSYNCNOTIFY_IPC_NAME, PSYNCNOTIFY_CONFLICT_SAVE_FILE, (unsigned char*)(filename.toLocal8Bit().data()), filename.size());
                            } else {*/
                                managedNotifyActivated(i18n("File could not be save by present way"), filename, 0, m_timer, 1);
                            }
                        }
                        break;
                case PSYNCNOTIFY_CONFLICT:
                        // reading settings
                        readSettings();
                        // TODO: Parsing input packet
                        filename = QString::fromLocal8Bit(QByteArray((const char*)(m_pkt->getData()+sizeof(int)),(m_pkt->getDataLen()-sizeof(int))));
                        qDebug() << "filename = " << filename;
                        for(int i=0; i<4; ++i) {
                            mode_conflict |= m_pkt->getData()[i] << ((i&3)*8);
                        }
                        std::cout << "mode_conflict = " << std::hex << mode_conflict << std::endl;
                        switch (mode_conflict)
                        {
                            case 0:
                                managedNotifyActivated(i18n("File was changed on the server and locally"), filename, 0, m_timer, 0);
                                break;
                            case 1:
                                managedNotifyActivated(i18n("File was deleted from the server and from the local base"), filename, 1, m_timer, 0);
                                break;
                        }

                        break;
		default:
                qDebug("Unknown type of message. The number of message is %d", (int)m_pkt->getType());
        }
    }

    if (m_pkt)
        delete m_pkt;
}

// Tray button slot
//! Функция начала и завершения синхронизации
void PSyncNotify::syncronizeStartStop()
{
    IPCPacket* pkt = NULL;

    if (m_sync_is_running)
    {
        pkt = IPCPacket::Create(PSYNCD_SYNC_STOP, (char*)PSYNCNOTIFY_IPC_NAME,0,0);
    }
    else
    {
    	pkt = IPCPacket::Create(PSYNCD_SYNC_START, (char*)PSYNCNOTIFY_IPC_NAME,0,0);
    }

    if(m_us->writeDatagram((const char*)pkt->getBuffer(), pkt->getSize(), PSYNCD_IPC_NAME)>=0)
    {
    	syncrAction->setEnabled(false);
    }

    if (pkt) delete pkt;
}

//! Функция инициализации сокета приложения
void PSyncNotify::initSocket()
{
    m_us = new UnixSocket();
    if (m_us->init())
    {

        QString self_address(PSYNCNOTIFY_IPC_NAME);
        qDebug("sockfd = %d", m_us->getSockfd());
        QFileInfo fi(PSYNCNOTIFY_IPC_NAME);
        if (!fi.absoluteDir().exists())
        	fi.absoluteDir().mkdir(PSYNCNOTIFY_IPC_NAME);

        if (m_us->bind(self_address.toLocal8Bit().data()))
        {
            m_sn = new QSocketNotifier(m_us->getSockfd(), QSocketNotifier::Read);
            connect(m_sn , SIGNAL(activated(int )), this, SLOT(receiveNotification()));
        }
        else
        {
            qDebug("Error of local socket bind");
            QMessageBox::critical(referenceWidget, _PROGRAMM_NAME, i18n("The program error occurred. Information about the process and since the last synchronization may be displayed incorrectly"), QMessageBox::Ok);
        }
    }
    else
    {
        qDebug("Error of local socket init");
        QMessageBox::critical(referenceWidget, _PROGRAMM_NAME, i18n("The program error occurred. Information about the process and since the last synchronization may be displayed incorrectly"), QMessageBox::Ok);
    }
}

//! Функция установки всплывающей подсказки в исходное состояние, после отображения в ней процесска загрузки файлов
void PSyncNotify::setToolTipStopLoadFile()
{
    setToolTipIconByName("sync");   //
    setToolTipTitle(_SERVICE_NAME); // TODO: percent tool tip
    setToolTipSubTitle(QString());  //
}

//! Функция установки меню приложения в исходное состояние, после остановки процесса синхронизации
void PSyncNotify::restoreAfterStopSync()
{
    m_sync_is_running = false;
    m_gif->stop();
    setToolTipStopLoadFile();
    syncrAction->setText(i18n("Synchronize"));
    syncrAction->setEnabled(true);
    QTimer::singleShot(500,this,SLOT(updateLastSyncTimeLabel()));
    setIconByPixmap(QIcon(":synchronize.png").pixmap(24,24));
    lastsynctimeAction->setVisible(true);
}

void PSyncNotify::singleModeSaveFile()
{
    callFileDialog(0);
}

void PSyncNotify::callFileDialog(int opt)
{
    //IPCMedia::send(PSYNCCONFIG_IPC_NAME, PSYNCNOTIFY_IPC_NAME, PSYNCNOTIFY_FILEDIALOG_STATE);
    IPCMedia::send(PSYNCD_IPC_NAME, PSYNCNOTIFY_IPC_NAME, PSYNCD_FILEDIALOG);
    notify_timer->stop();

    QString sf;     // TODO: del this vriable and to do with QStringList

    QFileDialog *dialog = new QFileDialog(referenceWidget, i18n("Folder to save files by default"));
    dialog->setAcceptMode(QFileDialog::AcceptOpen);
    dialog->setDirectory(QDir::homePath());
    dialog->setFileMode(QFileDialog::Directory);
    dialog->setOption(QFileDialog::DontUseNativeDialog,true);
    dialog->setLabelText(QFileDialog::FileName, i18n("Name: "));
    dialog->setLabelText(QFileDialog::FileType, i18n("Filter: "));
    dialog->setLabelText(QFileDialog::LookIn, i18n("Places")); // TODO: Look in
    dialog->setLabelText(QFileDialog::Accept, i18n("OK"));
    dialog->setLabelText(QFileDialog::Reject, i18n("Cancel"));

    dialog->setWindowIcon(QIcon(":synchronize.png").pixmap(128,128)); // TODO: should stay 128x128 pixel

    QStringList fileNames;
    if (dialog->exec()) fileNames = dialog->selectedFiles();
        if (!fileNames.isEmpty())
            sf = fileNames[0];
        else {
            //IPCMedia::send(PSYNCCONFIG_IPC_NAME, PSYNCNOTIFY_IPC_NAME, PSYNCNOTIFY_FILEDIALOG_STATE);
            refuseSaveFile();
            return;                 // TODO: answer only to deamon
        }

    QFileInfo fi(sf);

    while (!fi.isWritable()) {
        QMessageBox::critical(referenceWidget, _SERVICE_NAME, i18n("It is impossible to write down files in a folder. Please, choose a writable folder."), QMessageBox::Ok);
        dialog->selectFile(QDir::homePath());
        fileNames.clear();
        if (dialog->exec()) fileNames = dialog->selectedFiles();
            if (!fileNames.isEmpty())
                sf = fileNames[0];
            else {
                //IPCMedia::send(PSYNCCONFIG_IPC_NAME, PSYNCNOTIFY_IPC_NAME, PSYNCNOTIFY_FILEDIALOG_STATE);
                refuseSaveFile();
                return;             // TODO: answer only to deamon
            }
        fi.setFile(sf);
    }

    if (isRecursiveSaving && opt)
        recursiveMap.insert(tmp_string, sf);

    //IPCMedia::send(PSYNCCONFIG_IPC_NAME, PSYNCNOTIFY_IPC_NAME, PSYNCNOTIFY_FILEDIALOG_STATE);
    qDebug() << "timer = " << m_timer;
    qDebug() << "(psyncnotify) dir ame = "  << sf;
    IPCMedia::send(PSYNCD_IPC_NAME, PSYNCNOTIFY_IPC_NAME, PSYNCD_CONFLICT_SAVE_FILE, (unsigned char*)(sf.toLocal8Bit().data()), sf.size());
    //return sf;

}

void PSyncNotify::recursiveSave()
{
    isRecursiveSaving = true;
    callFileDialog(1);
}

void PSyncNotify::refuseSaveFile()
{
    unsigned char answer[1] = { 0x30 };
    IPCMedia::send(PSYNCD_IPC_NAME, PSYNCNOTIFY_IPC_NAME, PSYNCD_CONFLICT_SAVE_FILE, answer,sizeof(unsigned char));
}

//! Функция отправки сообщения модулю psyncd, в результате выбора первого действия на уведомлении о разрешении конфликтов
void PSyncNotify::sendConflictFirst()
{
    unsigned char answer[1] = { 0x31 };
    IPCMedia::send(PSYNCD_IPC_NAME, PSYNCNOTIFY_IPC_NAME, PSYNCD_CONFLICT, answer,sizeof(unsigned char));
    // TODO:
}

//! Функция отправки сообщения модулю psyncd, в результате выбора второго действия на уведомлении о разрешении конфликтов
void PSyncNotify::sendConflictSecond()
{
    unsigned char answer[1] = { 0x32 };
    IPCMedia::send(PSYNCD_IPC_NAME, PSYNCNOTIFY_IPC_NAME, PSYNCD_CONFLICT, answer,sizeof(unsigned char));
    // TODO:
}

//! Функция отправки сообщения модулю psyncd, в результате выбора третьего действия на уведомлении о разрешении конфликтов
void PSyncNotify::sendConflictThird()
{
    unsigned char answer[1] = { 0x30 };
    IPCMedia::send(PSYNCD_IPC_NAME, PSYNCNOTIFY_IPC_NAME, PSYNCD_CONFLICT, answer,sizeof(unsigned char));
    // TODO:
}

//! Функция открытия модуля настроек (приложение psyncconfig)
void PSyncNotify::openSettings()
{
    m_na.remove("test2");
    KProcess* process = new KProcess();
    *process << QString(GLOBAL_EXECUTABLE_NAME_CONFIG);
    //process->setProgram(QString("kcmshell4"), QStringList("kcm_sync"));
    process->start();
}

//! Функция чтения настроек из файла конфигурации
void PSyncNotify::readSettings()
{
//    int size_value=512;
//    char value[size_value];

    Lconfig conf(GLOBAL_SERVICE_LCONFIG_DIR); //config object
    if(conf.readConfig()!=0)   std::cerr <<__FILE__<<" "<<__LINE__<< " Error working with config!"<< std::endl; 

    m_show_tray_icon = conf.readInt(LSETTINGS_SHOW_TRAY);
    if(m_show_tray_icon == -1) m_show_tray_icon = true;

    m_show_start_end_notifications = conf.readInt(LSETTINGS_SHOW_NOTIFY_START_END);
    if(m_show_start_end_notifications == -1) m_show_start_end_notifications = true;

    m_show_file_notifications= conf.readInt(LSETTINGS_SHOW_NOTIFY_FILE);
    if(m_show_file_notifications == -1) m_show_file_notifications= true;

    m_sync_on= conf.readInt(LSETTINGS_SYNC_IS);

    int timer=0;
    timer = conf.readInt(LSETTINGS_CONFLICT_TIMER);
    if (timer!= -1)  m_timer = timer;
    else             m_timer = 10;
}

//! Функция обновления сведений о времени последней синхронизации
void PSyncNotify::updateLastSyncTimeLabel()
{
    unsigned int time=0;
    QString last_sync_time= i18n("Unknown");

    Lconfig conf(GLOBAL_SERVICE_LCONFIG_DIR); //config object
    if(conf.readConfig()!=0)   std::cerr <<__FILE__<<" "<<__LINE__<< " Error working with config!"<< std::endl; 
    time = conf.readInt(LSETTINGS_LAST_SYNC_TIME);

    if((int)time != -1 && (int)time != 0)
    {
    	QDateTime dt;
    	dt.setTime_t(time);
    	KLocale *aLocale = KGlobal::locale();
    	last_sync_time = aLocale->formatDateTime(dt, KLocale::FancyShortDate).toLower();
    };

    setLastSyncTimeLabel(i18n("The last synchronization: ") + last_sync_time);
}

//! Функция установки отображения времени последней синхронизации в меню
void PSyncNotify::setLastSyncTimeLabel(QString str)
{
        //QLabel *label = (QLabel*)lastsynctimeAction->defaultWidget();
        //label->setText(str);
    lastsynctimeAction->setText(str);
}

//! Функция обновления иконки синхронизации
void PSyncNotify::updateIcon()
{
    setIconByPixmap(m_gif->currentPixmap());
    setToolTipIconByPixmap(m_gif->currentPixmap()); // TODO: percent tooltip
    setToolTipTitle(_SERVICE_NAME);                 // TODO: percent tooltip
}

//! Функция вызова уведомления о начале синхронизации
void PSyncNotify::reportSyncStart()
{
	QString time_string = KGlobal::locale()->formatDateTime(QDateTime::currentDateTime(), KLocale::FancyShortDate);
        generalNotify(_NOTIFICATION_CAPTION, i18n("%1 : Synchronization start! ",time_string), QIcon(":synchronize.png").pixmap(32,32), referenceWidget);
}

//! Функция вызова уведомления о завершении синхронизации
void PSyncNotify::reportSyncStop()
{
	QString time_string = KGlobal::locale()->formatDateTime(QDateTime::currentDateTime(), KLocale::FancyShortDate);
        generalNotify(_NOTIFICATION_CAPTION, i18n("%1 : Synchronization end!",time_string), QIcon(":synchronize.png").pixmap(32,32), referenceWidget);
}
