//! \file WebDAV.cc Реафлизация класса работы с WebDav сервером
#include <string>
#include <fstream>
#include <sstream>
#include <cstdlib>
#include <errno.h>
#include <stdio.h>
#include <memory.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <time.h>
#include <map>
#include <dirent.h>
#include "WebDAV.h"
#include "http_hdr.h"
#include "tinyxml.h"

#include "../liblcfg/include/lconfig.h"

//! Статическая функция очистки ХМЛ вывода от namespace'ов 
/**
 * \param doc указатель на верх хмл дерева для очистки
 */
static void NamespaceClear(TiXmlNode* doc)
{
	if(!doc)
		return;
	string tmp;
	TiXmlNode* pChild;
	tmp=doc->ValueStr();
	if(tmp.find(":")!=string::npos){
		doc->SetValue(tmp.substr(tmp.find(":")+1,string::npos));
	}
	for ( pChild = doc->FirstChild(); pChild != 0; pChild = pChild->NextSibling())
		NamespaceClear(pChild);

}
using namespace std;

//! Конструктор класса
WebDav::WebDav()
{
	pthread_spin_init(&tlock,0);
}
     
//! Деструктор класса
WebDav::~WebDav()
{
	pthread_spin_destroy(&tlock);
}
//Приватные члены)
//Файлы
//! Получение sTag'a с сервера
/**
 * \param uname путь на сервере
 * \return sTag  или пустая строка при неудаче
 */
string WebDav::getFileSTag(string uname)
{
	return propfind(uname,string(),"getstag");
}
//! Блокировка файла на сервере
/**
 * \param uname путь на сервере
 */
int WebDav::Lock(string uname)
{
	int ret=0;
	pthread_spin_lock(&tlock);
#ifndef NOLOCK
	try{
		ret=lock(uname, lock_token);
	}catch(http_exception::not_found){
	}catch(http_exception::error err){
		if(err.status()!=500)
			throw;
	}
#endif
	return ret;
}

//! Разблокировка файла на сервере
/**
 * \param uname путь на сервере
 */
int WebDav::Unlock(string uname)
{
	int ret=0;
#ifndef NOLOCK
	try{
		ret=unlock(uname, lock_token);
	}catch(...){};
	if(ret){
		cerr<<"UNLOCK error:"<<ret<<endl;
	}
#endif
	pthread_spin_unlock(&tlock);
	lock_token.clear();
	return ret;
}
//! Удаление файла на сервере
/**
 * \param path Путь на сервере
 */
int WebDav::Delete(const string path)
{
	string p1=path;
	WebDav::_delete(path);
	while(true){
		if(p1.rfind("/")==string::npos)
			break;
		p1=p1.substr(0,p1.rfind("/"));	
		if(!(GetServerList(p1+"/").empty()))	//Если папка не пустая
			break;
		if(GetServerState(p1+"/").valid())		//Если папка в синхронизации
			break;
		WebDav::_delete(p1+"/");
	}
	return 0;
}

//! Установка метаданных для файла на сервере
/**
 * \param uname путь к объекту на сервере
 * \param meta ассоциативный массив метаданных
 */
void WebDav::SetMeta(string uname,map<string, string> meta)
{
	setprop(uname, meta, lock_token);
}
//! Получение режима файла с сервера
/**
 * \param uname путь к объекту на сервере
 * \param mode  указатель на структуру для записи режима файла
 */
int WebDav::GetMode(string uname,mode_t* mode)
{
	string ret=propfind(uname, lock_token,"filemode");
	stringstream s(ret);
	s>>*mode;
	return last_error;
}

//! Получение sTag'a с сервера через указатель в параметрах
/**
 * \param uname путь на сервере
 * \param sTag  sTag или пустая строка при неудаче
 */
int WebDav::GetSTag(string uname,string* sTag)
{
	*sTag = propfind(uname, lock_token,"getstag");
	return last_error;
}
//! Получение Content-length с сервера
/**
 * \param uname путь на сервере
 * \param cl указатель на переменную типа unsigned
 */
int WebDav::GetCL(string uname,unsigned* cl)
{
	string sCL = propfind(uname, lock_token,"getcontentlength");
	istringstream s(sCL);
	s>>*cl;
	return last_error;
}

//! Создание каталога на сервере
/**
 * \param path путь к файлу на сервере
 */
int WebDav::Mkcol(const string path)
{
	try{
		mkcol(path, lock_token);
	}catch(http_exception::conflict){
		Create_path(path);
		return 0;
	}catch(http_exception::not_allowed){		//Каталог уже есть - пропускаем
	}catch(http_exception::error e){
		cerr<<"ошибка создания файла на сервере:"<<e.status()<<endl;
		throw;
	}
	return 0;
}
//! Создание пути до файла на сервере
/**
 * \param uname путь на сервере
 */
int WebDav::Create_path(const string uname)
{
	string path=uname.substr(0,uname.rfind("/")+1);
	string::size_type i=0;
	while(true){
		i=path.find("/",i+1);
		if(i == string::npos)
			break;
		string p1=path.substr(0,i);
		try{
			Mkcol(p1);
		}catch(http_exception::moved){
		}
	}
	return 0;
}
//! Отправка файла на сервер
/**
 * \param uname путь на сервере
 * \param path путь на клиенте
 */
int WebDav::Put(string uname, string path)
{
	try{
                neon_operation = true;
		try_put(uname, lock_token);
		put(path, uname, lock_token);
                neon_operation = false;
	}catch(http_exception::forbidden){
		Create_path(uname);
		put(path, uname, lock_token);
		return 0;
	}catch(http_exception::error e){
		cerr<<"ошибка создания файла на сервере:"<<e.status()<<endl;
		throw;
	}
//	setprop(uname, "clientpath", path, lock_token);
	return 0;
}

//! Получение файла с сервера
/**
 * \param uname путь на сервере
 * \param path путь на клиенте
 */
int WebDav::Get(string uname, string path)
{
    neon_operation = true;
	get_file_size(path, uname, lock_token);
	int ret=get(path, uname, lock_token);
    neon_operation = false;
	return ret;
}
//! Получение экземпляра объекта sync_object на основе данных с сервера
/**
 * \param uname путь на сервере
 * \return объект sync_object
 */
sync_object WebDav::GetServerState(string uname)
{
	string _uname;
	string path;			
	mode_t mode;
	string sTag;
	char level;
	unsigned cl;

	string allprop=propfind(uname, lock_token ,string());
	//XML
	TiXmlDocument ret_request;
	ret_request.Parse(allprop.c_str());
	NamespaceClear(ret_request.RootElement());
	//TO multistatus
	TiXmlNode* parent=ret_request.FirstChild("multistatus");
	while(parent){
		TiXmlNode* child = parent->FirstChild("response");
		TiXmlHandle i(child);
		TiXmlElement* element;
		//Путь к файлу на сервере
		element = i.FirstChild("href").ToElement();
		if(!element)
			break;
		_uname = element->GetText();
		_uname = _uname.substr(conn_ctx.path.length());				//Обрезаем до имени файла	
		_uname = http_hdr::from_persent_byte(_uname);
		if(uname!=_uname)
			break;
		//Путь на клиенте
		element = i.FirstChild("propstat").FirstChild("prop").FirstChild("clientpath").ToElement();
		if(!element)
			path=string();
		else
			path = element->GetText();
		//Етаг
		element = i.FirstChild("propstat").FirstChild("prop").FirstChild("getstag").ToElement();
		if(!element)
			sTag=string();
		else
			sTag = element->GetText();
		//Mode
		element = i.FirstChild("propstat").FirstChild("prop").FirstChild("filemode").ToElement();
		if(!element)
			mode=0;
		else{
			istringstream s(element->GetText());
			s>>mode;
		}
		//level
		element = i.FirstChild("propstat").FirstChild("prop").FirstChild("level").ToElement();
		if(!element)
			level=-1;
		else{
			istringstream s(element->GetText());
			s>>level;
		}
		//CL
		element = i.FirstChild("propstat").FirstChild("prop").FirstChild("getcontentlength").ToElement();
		if(!element)
			cl=0;
		else{
			istringstream s(element->GetText());
			s>>cl;
		}
		break;
	}
//cout<<"Path local - "<<path<<"\n";
//	path = "/" + _uname;
//	if(path.rfind("/") == path.length()-1)
//		path.erase(path.length()-1,1);
	sync_object so(_uname, path, 0, sTag, mode, -1, 0, cl);
	if(level == 1)
		so.leveldown();
	return so;
}
//! Получение вектора объектов синхронизации, дочерних к uname на сервере
/**
 * \param uname путь на сервере
 * \return вектор объектов синхронизации
 */
vector<sync_object> WebDav::GetServerList(string uname)
{
	vector<sync_object> ret;
	string allprop=propfind(uname, lock_token ,string());
	//XML
	TiXmlDocument ret_request;
	ret_request.Parse(allprop.c_str());
	NamespaceClear(ret_request.RootElement());
	//TO multistatus
	TiXmlNode* parent=ret_request.FirstChild("multistatus");
	if(parent){
		TiXmlNode* child=0;	
		while((child = parent->IterateChildren("response",child))!=NULL){			//TODO:Возможно контролировать по response/propstat/status?
			string _uname;
			string path;
			mode_t mode;
			string sTag;
			char level;
			unsigned cl;
			TiXmlHandle i(child);
			TiXmlElement* element;
			//Путь к файлу на сервере
			element = i.FirstChild("href").ToElement();
			if(!element)
				continue;
			_uname = element->GetText();
			_uname = _uname.substr(conn_ctx.path.length());				//Обрезаем до имени файла	
			_uname = http_hdr::from_persent_byte(_uname);
			if(uname==_uname)
				continue;
			//Путь на клиенте
			element = i.FirstChild("propstat").FirstChild("prop").FirstChild("clientpath").ToElement();
			if(!element)
				path=string();
			else
				path = element->GetText();
			//Етаг
			element = i.FirstChild("propstat").FirstChild("prop").FirstChild("getstag").ToElement();
			if(!element)
				sTag=string();
			else
				sTag = element->GetText();
			//Mode
			element = i.FirstChild("propstat").FirstChild("prop").FirstChild("filemode").ToElement();
			if(!element)
				mode=0;
			else{
				istringstream s(element->GetText());
				s>>mode;
			}
			//level
			element = i.FirstChild("propstat").FirstChild("prop").FirstChild("level").ToElement();
			if(!element)
				level=-1;
			else{
				istringstream s(element->GetText());
				s>>level;
			}
			//CL
			element = i.FirstChild("propstat").FirstChild("prop").FirstChild("getcontentlength").ToElement();
			if(!element)
				cl=0;
			else{
				istringstream s(element->GetText());
				s>>cl;
			}
//cout<<"Path local - "<<path<<"\n";
//			path = "/" + _uname;
//			if(path.rfind("/") == path.length()-1)
//				path.erase(path.length()-1,1);
			sync_object so(_uname, path, 0, sTag, mode, -1, 0,cl);
			if(level == 1) 
				so.leveldown(); 
			ret.push_back(so);
		}
	}
	return 	ret;
}
//! Попытка аутенфикации
/**
 * Проверяет квоту, если происходит какая-либо ошибка - возбуждает исключение
 */
int WebDav::TryAuth()
{
	long long max, cur;
	if(!GetQuote(max, cur)){
            	string path_cfg = conn_ctx.home+"sync.cfg";
                Lconfig cfg(path_cfg); //config object

                if(cfg.readConfig()!=0) {
            		std::cerr <<__FILE__<<" "<<__LINE__<< " Error working with config!"<< std::endl;
            		return -1;
            	}

                cfg.writeLongInt("cfg.info.quota_total", max);
                cfg.writeLongInt("cfg.info.quota_used", cur);

                if(cfg.writeConfig()!=0) {
            		std::cerr <<__FILE__<<" "<<__LINE__<< " Error working with config!"<< std::endl; 
            		return -1;
            	}
	}
	return 0;
}
//! Получение квоты теущего пользователя на сервере
/**
 * \param max ссылка на переменную максимального объема пользователя на сервере
 * \param cur ссылка на переменную текущего объема, занятого пользователем на сервере
 */
int WebDav::GetQuote(long long& max, long long& cur)
{
	string allprop=propfind(string(),string(),string());
	//XML
	TiXmlDocument ret_request;
	ret_request.Parse(allprop.c_str());
	NamespaceClear(ret_request.RootElement());
	TiXmlHandle handle(&ret_request);
	TiXmlElement* child=handle.FirstChild("multistatus").FirstChild("response").FirstChild("propstat").FirstChild("prop").FirstChild("quota-available-bytes").ToElement();
	if(!child){
		return last_error; }
	max =atoll(child->GetText());
	child=handle.FirstChild("multistatus").FirstChild("response").FirstChild("propstat").FirstChild("prop").FirstChild("quota-used-bytes").ToElement();
	if(!child){
		return last_error;}
	cur =atoll(child->GetText());
	return 0;
}
