//! \file WebDAV.h Описание класса работы с WebDav сервером
#ifndef WEBDAV_H
#define WEBDAV_H

#include <string>
#include <map>
#include <vector>
#include "WebDav_p.h"
#include "WebDav_neon.h"
#include "main.h"
#include "sync_obj.h"

using namespace std;
#ifdef USE_LIBNEON
class WebDav: public WebDav_neon 	//!< Класс работы с WebDav сервером на основе neon
#else
class WebDav: public WebDav_private	//!< Класс работы с WebDav сервером на основе http_hdr.cc
#endif
{
public:
	WebDav();
	~WebDav();
private:
	pthread_spinlock_t tlock;	//!< Блокировка класса
public:
	string getFileSTag(string uname);
	unsigned last_error;		//!< Последняя ошибка при выполнениие запроса(TODO:не используется, удалить)
	void SetMeta(string uname, map<string, string> meta);
	int GetMode(string uname,mode_t* mode);
	int GetSTag(string uname,string* sTag);
	int GetCL(string uname,unsigned* cl);
	
	int Create_path(const string uname);

	int Mkcol(const std::string path);
	int Delete(const std::string path);
	int Get(string uname, string path);
	int Put(string uname, string path);
	int Lock(string uname);
	int Unlock(string uname);
	int Options();			//!< Не реализовано
	int TryAuth();
	int GetQuote(long long &max, long long &cur);
	vector<sync_object> GetServerList(string uname=string());
	sync_object GetServerState(string uname);
};
#endif //WEBDAV_H
