//! \file http_hdr.h Заголовочный файл класса работы с HTTP пакетами и исключениями
#include <string>
#include <fstream>
#include <map>
#include <openssl/ssl.h>
//#define USE_LIBNEON
namespace http_exception
{
	//! Класс исключения сетевого взаимодействия
	class error{
			int err;	//!<Номер ошибки
		public:
			//! Конструктор класса
			//! \param e код ошибки
			error(int e):err(e){};	
			int status() const {return err;} //!< Получение номера ошибки
		static  void check(int err);	//!< статический метод возбуждения исключения при HTTP ошибках
	};
	//! Класс исключения 404 - Not Found
	struct not_found : public error {not_found():error(404){};};
	//! Класс исключения 423 - Locked
	struct locked : public error {locked():error(423){};};
	//! Класс исключения 405 - Not Allowed
	struct not_allowed : public error {not_allowed():error(405){};};
	//! Класс исключения 301 - Moved
	struct moved : public error {moved():error(301){};};
	//! Класс исключения 401 - Autenfication Required
	struct auth_err : public error {auth_err():error(401){};};
	//! Класс исключения 403 - Forbidden
	struct forbidden : public error {forbidden():error(403){};};
	//! Класс исключения 409 - Conflict
	struct conflict : public error {conflict():error(409){};};

	//! Класс исключения остановки синхронизации пользователем
	struct user_stop{user_stop(){};};
	//! Класс исключения нехватки свободного места
	struct no_space{no_space(){};};
}


//!Класс работы с HTTP - запросами
class http_hdr
{
public:
static  std::string from_persent_byte(std::string in);
#ifndef USE_LIBNEON
	public:
#else
	private:
#endif
	//! Код HTTP команды
	enum http_command
	{
		CM_MKCOL,
		CM_PUT,
		CM_PPATCH,
		CM_GET,
		CM_LOCK,
		CM_UNLOCK,
		CM_PFIND,
		CM_DELETE,
	};
	//! Код режима работы класса http_hdr
	enum work_mode
	{
		CT_NORMAL,
		CT_FILEIN,
		CT_FILEOUT,
	}  mode;	//!<Режим работы класса
	//! Коды возврата операций с протоколом
	enum proto_ret 
	{
		HH_OK=0,
		HH_RETRY,
		HH_FAIL,
	};
	//! Состояние соединения с сервером
	enum state 
	{
		WD_DISCONNECTED,
		WD_CONNECTED,
		WD_EPIPE,
	}
	conn_state;	//!< Cостояние соединения с сервером



	http_hdr();
	~http_hdr();
	http_hdr(http_command command, std::string filename);
	void add_body(const void* buf, size_t len);
	void parse_rcvd(std::string _s);
	void add_lt(std::string lock_token);
	void add_file_body(std::ifstream* fd, size_t size);
	//! Установка файлового потока fd как потока, получающего данные  из HTTP ответа
	inline void add_file_rbody(std::ofstream* fd){ofd=fd; mode=CT_FILEOUT;}  

	void connect();
	void disconnect();				//!< Не используется
	int check_cert(SSL* conn,const char* host);

	int process_request();
	proto_ret send();
	proto_ret recv();
	
	int pending();
	int _send(const void* buf, size_t len);
	int _recv(void* buf, size_t len);

	std::string toString();

	//FILES
	std::string fname;				//!< Имя файла
	std::ifstream* ifd;				//!< Указатель на входной файловый поток
	std::ofstream* ofd;				//!< Указатель на выходной файловый поток
	//IN
	std::string description;		//!<	Расшифровка статуса
	int status;				//!<	Числовой статус ответа
	unsigned  more;				//!<	Количество докачиваемых данных
	char* rdata;				//!<    Указатель на принятые данные
	char* more_data;			//!<    Указатель на докачку данных
	unsigned  cl;				//!<	Размер данных в запросе
	unsigned  more_data_size;		//!<	Размер уже принятых данных
	std::map<std::string,std::string> ivalues;	//!<	Принимаемые значения
	std::string::size_type end;		//!<	Конец ХТТП заголовка и начало данных
	//OUT
	std::string header;			//!<	Отправляемый запрос верхняя строчка
	unsigned  data_size;			//!<	Размер отправляемых данных
	char* data;				//!<	Отправляемые данные
	std::map<std::string,std::string> values;	//!< Отправляемые заголовки

	int sock;				//!<	Дескриптор сокета
	//SSL
	SSL_CTX* ssl_ctx;			//!<	SSL контекст
	SSL* ssl;				//!<	Указатель на объект SSL
	BIO* bio_err;				//!< 	Указатель на структуру ошибки ввода/вывода
	BIO* sbio;				//!< 	Указатель на структуры ввода/вывода SSL

};


