//! \file sync_obj.cc Объявление класса объекта синхронизации
#ifndef FILE_SYNC_H 
#define FILE_SYNC_H
#include <map>
#include <string>
#include <sstream>
#include <sys/types.h>

using namespace std;
class filedb;
class WebDav;
//! Класс, инкапсулирующий объект синхронизации  
class sync_object
{
public:
	//! Флаги синхронизации
	enum files_flags    
	{
		FL_DELETE=1,		//!< Объект помечен к удалению
		FL_DELETED=2,		//!< Объект удален
		FL_LEVEL2=4,		//!< Объект синхронизации 2-го уровня
		FL_SYNCRONIZED=8,	//!< Объект был синхронизирован с сервером
	};
private:
	//! Состояние обновления объекта
	enum upd_state
	{
		SO_UPTODATE,		//!< Синхронизирован
		SO_CREATE,		//!< Помечен к созданию на сервер
		SO_PUSH,		//!< Помечен к переносу на сервер
		SO_PULL,		//!< Помечен к переносу с сервера
		SO_DELETE,		//!< Помечен к удалению
		SO_LCREATE,		//!< Создание у локального пользователя
		SO_CONFLICT,		//!< Конфликт 
	};
	//! Состояние конфликта
	enum cnfl_state
	{
		CONF_MOD_ALLSIDE=0,	//!< Файл изменен с обеих сторон
		CONF_DEL_ALLSIDE,	//!< Файл удален с обеих сторон
	};

	std::string uname;	//!<уникальное имя для хранения на сервере 
	std::string path;	//!<путь на локальной системе
	std::string sTag;       //!<sТаг на сервере
	time_t sync_time;	//!<время синхронизации
	mode_t mode;		//!<Права файла
	char sync_point;	//!<Принадлежность к точке синхронизации
	char flags;		//!<Флаги file_flags

	bool isCheckSubDir; 	//!< флаг отмены синхронизации объектов второго уровня

	size_t server_size;	//!< размер файла на сервере

	int inotify_fd;		//!< Дескриптор inotify
	int state;		//!< Состояние обновления
	filedb* db;		//!< Указатель на БД

	int get(WebDav* dav);	
	int put(WebDav* dav);
	void genSTag();
	time_t GetMTime();

	int put_subdir(string path, string uname, WebDav* dav); //!< не используется
	int get_subdir(string path, string uname, WebDav* dav); //!< не используется
	int checkSubdir(WebDav* dav);
	sync_object* getUpDir();

	void checkState(WebDav* dav);

	map<string,string> getMetaMap() const;
public:
	sync_object(string path, string uname);
	sync_object(string _uname, string _path,time_t _sync_time, string _stag,
			short _mode, char _sync_point, char _flags, size_t serversize=0);

	sync_object(string _l);
	sync_object();
	~sync_object();

	//! Установка указателя на БД
	void setDB(filedb* _db){db=_db;}
	void leveldown(){flags|=FL_LEVEL2;}	//!< Установка второго уровня синхронизации объекта
	int level() const {return flags&FL_LEVEL2;} 	//!< Получение уровня синхронизации объекта
	void set_ifd(int fd) {inotify_fd=fd;}	//!< Установка дескриптора inotify
	int get_ifd() {return inotify_fd;}	//!< Получение дескриптора inotify
	int valid();

	int operator==(string s);
	int operator==(sync_object s);
	bool inDir(string path);

	int update(WebDav* dav);
static  void mkdirto(string path);

	void getData(string& uname, string& path,time_t& sync_time, string& stag, mode_t& mode, char& sync_point, char& flags) const; 
	string getStringData() const;
	string getPath() const {return path;}  //!< Получение локального 
	string getUName() const	{return uname;}//!< Получение пути на сервере

	void setPath(string lpath)  { path = lpath;}
	void replacePath(string old_path, string new_path);

	void setDelete();
	//! Отмена удаления объекта
	void undelete(){flags^=FL_DELETE;}
	bool toDelete(){return flags&FL_DELETE;}	//!< Проверка, отмечен ли объект к удалению
	bool canDelete(){return flags&FL_DELETED;}	//!< Проверка, удален ли объект

	bool isSyncronized(){return flags&FL_SYNCRONIZED;}	//!< Проверка, синхронизирован ли объект
};

#endif
