//! \file main.cc Реализация главной функции приложения
//! \mainpage Демон синхронизации MandrivaSync
#include <stdio.h>
#include <iostream>
#include <fstream>
#include <sstream>
#include <stdlib.h>
#include <signal.h>
#include <sys/poll.h>
#include <errno.h>
#include <sys/stat.h>
#include "sync.h"
#include "http_hdr.h"
#include "ipc_interface.h"
#include "ipc_interface_local.h"
#include "ipc_proxy.h"
#include "../liblcfg/include/lconfig.h"

#define THREAD_STOP_TIMEOUT 1000

#define HOME_ENV "HOME"//!< Название переменной окружения $HOME
using namespace std;
http_ctx conn_ctx;		//!< Глобальный  контекст настроек соединения
static int work=1;		//!< Флаг работы главного цикла приложения 

int neon_counter;
bool neon_operation = false;

//! Обработчик сигнала SIGTERM 
void term_hndlr(int a)		
{	
	work=0;
	cerr<<"Завершение работы"<<endl;
}

int check_config(string c_path)
{
    bool bad_file = false;
    struct stat st;
    if(stat(c_path.c_str(), &st)!=0)  bad_file = true;
    else  {
    	if(st.st_size == 0)  bad_file = true;
    }
    if(bad_file) {
    	string dir = c_path.substr(0,c_path.rfind("/"));
    	mkdir(dir.c_str(), 0775);
    	string command = "/bin/cp /etc/skel/.psyncclient/sync.cfg " + c_path.substr(0,c_path.rfind("/")+1);  // /home/user/444/sync.cfg
        	if(system(command.c_str()) != 0)
        		return -1;
    }
    return 0;
}

//! Главная функция приложения 
int main()
{
	try{
		//!Формируются пути к сокетам синхронизации
		char* he=getenv(HOME_ENV);
		if(!he)
			throw "Ошибка получения $HOME";
		string home(he);
		home+="/";home+=FOLDER_NAME;home+="/";
		string pathSync=home+SYNCRONIZE_PROGRAM_NAME;
		string pathNotify=home+PSYNCNOTIFY_IPC_NAME;
		string pathConfig=home+CONFIG_PROGRAM_NAME;
		string pathDolphin=home+DOLPHIN_PROGRAM_NAME;

		string path_cfg = home+"sync.cfg";
		if(check_config(path_cfg) != 0) {
			cerr<<"Not find config file!\n";
			return -1;
		}

		//! Создаются классы
		Lconfig conf(path_cfg);    //config object

		if(conf.readConfig()!=0) {
			std::cerr <<__FILE__<<" "<<__LINE__<< " Error working with config!"<< std::endl;
    		return -1;
		}

		string address, port, ssl, login, password;

		// Restore reqired parameters if needed
		address = conf.readString("cfg.server.address");
		if(address.empty()) conf.writeString("cfg.server.address", "sync1.mandrivasync.com");
		port = conf.readString("cfg.server.port");
		if(port.empty()) conf.writeString("cfg.server.port", "443");
		ssl = conf.readString("cfg.server.ssl");
		if(ssl.empty()) conf.writeString("cfg.server.ssl", "yes");
 
		if(conf.writeConfig()!=0) {
			std::cerr <<__FILE__<<" "<<__LINE__<< " Error working with config!"<< std::endl;
			return -1;
		}

    	// Dump working configuration (for DEBUG purposes only)
        string tmp_str;
        tmp_str = conf.readString("cfg.server.address");  cout<<"server address = "<<tmp_str<<"\n";
        tmp_str = conf.readString("cfg.server.port");     cout<<"server port = "<<tmp_str<<"\n";
        tmp_str = conf.readString("cfg.server.ssl");      cout<<"server ssl = "<<tmp_str<<"\n";
        tmp_str = conf.readString("cfg.user.login");      cout<<"user login = "<<tmp_str<<"\n";
        tmp_str = conf.readString("cfg.user.password");   cout<<"user password = "<<tmp_str<<"\n";
        tmp_str = conf.readString("cfg.user.login");      cout<<"server path = "<<string("/HTTP/")+tmp_str+"/"<<"\n";

        login = conf.readString("cfg.user.login");
        password = conf.readString("cfg.user.password");

        conn_ctx.timeout = conf.readInt("cfg.user.timer");
        conn_ctx.host=address;
        conn_ctx.path=string("/HTTP/") + login + "/";
        conn_ctx.port=(unsigned short)atoi(port.c_str());
        if(ssl.empty() || ssl == "no")  conn_ctx.ssl=false;
        else             		conn_ctx.ssl=true;

        if(!login.empty())
        	conn_ctx.creds=login+":"+password;
        conn_ctx.home=home;
        conn_ctx.pathConfig=pathConfig;
        conn_ctx.pathNotify=pathNotify;

        unlink((home+"serverlog").c_str());
        ipc_proxy ipc(pathSync.c_str());

        unsigned tm=0;
        int sync_is = conf.readInt("cfg.user.sync_is");
        if(sync_is!= 0 )
        	tm = conf.readInt("cfg.user.interval");

		syncronizer Sync(address,(unsigned short)atoi(port.c_str()),string("/HTTP/")+login+"/");
		Sync.dbconnect(home+"sync.db");
		Sync.setSyncTime(tm);
		Sync.setIPC(&ipc);
		conn_ctx.ipc=&ipc;

		conn_ctx.path_rename = false;

		int mode=syncronizer::MODE_TIME;
		if(!conf.readIntLookup("cfg.user.mode", mode))
			 conf.writeInt("cfg.user.mode", mode);
		mode = conf.readInt("cfg.user.mode");
		Sync.setMode(mode);

		{
			struct sigaction act;
			act.sa_handler = term_hndlr;
			sigemptyset(&act.sa_mask);
			act.sa_flags=0;
			sigaction(SIGINT,&act,NULL);
			sigaction(SIGTERM,&act,NULL);
		}

		int r;
		int counter = 0;
		Sync.start();
		while(work)
		{

			if(neon_operation)
			{
				if(counter < neon_counter)	counter = neon_counter;
				counter++;
	//cout<<"Neon counter - "<<neon_counter<<"\n";
	//cout<<"Counter - "<<counter<<"\n";
				if((counter - neon_counter)>THREAD_STOP_TIMEOUT)
				{
					neon_operation = false;
					Sync.stop();
					cerr<<"sync thread stopped!\n";
					Sync.start();
					cerr<<"sync thread started!\n";
					ipc.send(PSYNCNOTIFY_HTTP_ERROR, PSYNCNOTIFY_IPC_NAME);
					ipc.send(PSYNCNOTIFY_HTTP_ERROR, PSYNCCONFIG_IPC_NAME);
				}
			}
			else  counter = 0;

			r=ipc.next();
			if(r)
			{
				if(r==-EAGAIN){
					poll(NULL,0,100);
					continue;
				}
				cerr<<"ipc_nextberr"<<endl;
				continue;
			}
			cerr<<"GET:"<<ipc.getState()<<endl;

			switch(ipc.getState())
			{
				case ipc_proxy::SYNC_START:
					{
						int res = ipc.check_connect(conn_ctx.host.c_str());
						if(res == 0)
							Sync.syncAll();
						else {
							ipc.send(PSYNCNOTIFY_HTTP_ERROR, PSYNCNOTIFY_IPC_NAME);
							ipc.send(PSYNCNOTIFY_HTTP_ERROR, PSYNCCONFIG_IPC_NAME);
						}
					}
					break;
				case ipc_proxy::SYNC_STOP:
					cerr<<"остановка синхронизации"<<endl;
					Sync.syncStop();
					break;
				case ipc_proxy::FILE_ADD:
					{
						cerr<<"Добавление файла"<<endl;
						istringstream files(string(ipc.getData()));
						string line;

						Sync.setFileOperation();
						Sync.stopAuto();

						while ( std::getline ( files, line ) ) {
							Sync.addFile(line.c_str());
							cerr<<" "<<line<<endl;
						}

						Sync.startAuto();
					}
					break;
				case ipc_proxy::SYNC_STATUS:
					if(Sync.is_work())
						ipc.send(PSYNCD_ANS_SYNC_STATUS_ONLINE, ipc.from().c_str());
					else
						ipc.send(PSYNCD_ANS_SYNC_STATUS_OFFLINE, ipc.from().c_str());
					break;

				case ipc_proxy::FILE_DEL:
					{
						cerr<<"Удаление файла"<<endl;
						istringstream files(ipc.getData());
						string line;

						Sync.setFileOperation();
						Sync.stopAuto();

						while ( getline ( files, line ) ) {
							Sync.removeFile(line);
						}

						Sync.startAuto();

					}
					break;
				case ipc_proxy::DIR_STATUS:
					{
						string fpath = ipc.getData();
						string ret = Sync.checkDir(fpath);
						ipc.send(PSYNCD_OBJECTS_LIST, ipc.from(), ret);;
					}
					break;
				case ipc_proxy::PWD_CHANGED:
				case ipc_proxy::REQ_LOGIN:
					{
						int result = ipc.check_connect(conn_ctx.host.c_str());
						if(result == 0) {
							cerr<<"Запрос логина пароля\n";
							conn_ctx.creds=ipc.getData();
							conn_ctx.path=string("/HTTP/")+conn_ctx.creds.substr(0,conn_ctx.creds.find(":"))+"/";
							try{
								Sync.TryAuth();
								ipc.send(PSYNCCONFIG_ACCOUNT_VALID, PSYNCCONFIG_IPC_NAME);
							}catch(http_exception::auth_err){
								ipc.send(PSYNCCONFIG_ACCOUNT_INVALID, PSYNCCONFIG_IPC_NAME);
							}catch(...){
								ipc.send(PSYNCCONFIG_ACCOUNT_ERROR, PSYNCCONFIG_IPC_NAME);
							}
						}
						else ipc.send(PSYNCCONFIG_ACCOUNT_ERROR, PSYNCCONFIG_IPC_NAME);
					}
					break;
				case ipc_proxy::SET_CHANGED:
					{
						   if(conf.readConfig()!=0) {
								   std::cerr <<__FILE__<<" "<<__LINE__<< " Error working with config!"<< std::endl;
								   break;
						   }

						   conn_ctx.host=conf.readString("cfg.server.address");
						   conn_ctx.path=string("/HTTP/")+conf.readString("cfg.user.login")+"/";
						   conn_ctx.port=(unsigned short)atoi(conf.readString("cfg.server.port").c_str());
						   string login, pass;
						   login = conf.readString("cfg.user.login");
						   pass = conf.readString("cfg.user.password");
						   conn_ctx.creds=login+":"+pass;
						   string ssl = conf.readString("cfg.server.ssl");
						   if(ssl.empty() || ssl == "no")  conn_ctx.ssl=false;
                                                        else                       conn_ctx.ssl=true;

						   conn_ctx.timeout = conf.readInt("cfg.user.timer");

						   unsigned tm=0;
						   int sync_is = conf.readInt("cfg.user.sync_is");
						   if(sync_is!= 0 )  tm = conf.readInt("cfg.user.interval");

						   Sync.setSyncTime(tm);
						   int mode;
						   mode = conf.readInt("cfg.user.mode");
						   Sync.setMode(mode);
					}
					break;
				case ipc_proxy::LIST_ALL:
					//ipc.send(ipc_proxy::DIR_STATUS, ipc.from().c_str(), Sync.listAll());
					ipc.send(PSYNCD_OBJECTS_LIST, ipc.from(), Sync.listAll());
					break;
				case ipc_proxy::CONFLICT:{
						istringstream iret(ipc.getData());
						iret>>conn_ctx.conflict_flag;
						break;
					}
				case ipc_proxy::FILE_CONFLICT:{
						conn_ctx.file_path = ipc.getData();
												conn_ctx.file_conflict_flag = false;
											conn_ctx.file_wait_flag = false;
						break;
					}
				case ipc_proxy::FILE_DIALOG:{
						conn_ctx.file_conflict_flag = false;
						conn_ctx.file_wait_flag = true;
						break;
					}

				default:
					break;
			}
		}
		Sync.stop();
        if(conf.readConfig()!=0) {
    		std::cerr <<__FILE__<<" "<<__LINE__<< " Error working with config!"<< std::endl; 
    		return -1;
    	}
        conf.writeString("cfg.server.address", conn_ctx.host);
        if(conf.writeConfig()!=0) {
        	std::cerr <<__FILE__<<" "<<__LINE__<< " Error working with config!"<< std::endl; 
    		return -1;
    	}
        return 0;
	}
	catch(const char* str)
	{
		cerr<<"Critical error: "<<str<<endl;
		return -1;
	}
}
