//! \file sync.cc Реализация класса контекста синхронизации
#include <sstream>
#include <pthread.h>
#include <sys/poll.h>
#include <sys/ioctl.h>
#include <sys/inotify.h>
#include <errno.h>
#include <iostream>
#include "sync.h"
#include "http_hdr.h"

#include "../liblcfg/include/lconfig.h"

extern http_ctx conn_ctx;
//! Конструктор класса
/**
 * \param srv Не используется 
 * \param port Не используется 
 * \param col Не используется 
 */
syncronizer::syncronizer(string srv,unsigned short port, string col)
{
	is_working=0;
	stopflag=0;
	automode=0;
//	pthread_spin_init(&lock,0);
	sync_time=0;
	def_synctime=0;
	mode=MODE_NONE;

	file_operation = false;

}
//! Деструктор класса
syncronizer::~syncronizer()
{
	stop();
	stopAuto();
//	pthread_spin_destroy(&lock);
}

//! Поток периодической синхронизации
static void* thread(void* arg)
{
	syncronizer* ctx=(syncronizer*) arg;
	ctx->fthread();
	return NULL;
}
//! Поток автоматической синхронизации
static void* athread(void* arg)
{
	syncronizer* ctx=(syncronizer*) arg;
	ctx->athread();
	return NULL;
}
//! Поток периодической синхронизации - член класса
/**
 * Если режим mode не равен MODE_NONE, то если таймер синхронизации не ноль,
 * каждую секунду он декрементируется, а когда становится равным 0, 
 * запускается синхронизация и он устанавливается в текущий период синхронизации
 *
 */
void syncronizer::fthread()
{
	while(is_working){
		if(mode==MODE_NONE){
			poll(0,0,100);
			continue;
		}
		if(sync_time>0){
			if(!(--sync_time))
				--sync_time;
			poll(0,0,1000);
		}else if(sync_time == 0)
			poll(0,0,100);
		else {
			sync_time=def_synctime;
			sync();
		}
	}
}
//! Статическая функция получения дерева объектов на сервере
/**
 * \param dav указатель на объекта соединения WebDav
 * \return вектор объектов синхронизации с сервера
 */
static vector<sync_object> parseServerTree(WebDav* dav)
{
	vector <sync_object> srv_files = dav->GetServerList();
	vector <sync_object> ret;
	while(!srv_files.empty())
	{
		sync_object so=srv_files.back();
		srv_files.pop_back();
		string uname=so.getUName();
		if(uname[uname.length()-1]=='/'){
			vector <sync_object> srv_files1 = dav->GetServerList(uname);
			srv_files.insert(srv_files.end(),srv_files1.begin(),srv_files1.end());
		}
		if(so.valid())
			ret.push_back(so);
	}
	return ret;
}

//! Функция синхронизации всех файлов в базе и на сервере
int syncronizer::sync()
{
	stopAuto();
	WebDav dav;
	conn_ctx.stopflag=0;
	string path_cfg = conn_ctx.home+"sync.cfg";
        Lconfig cfg(path_cfg); //config object
        if(cfg.readConfig()!=0) {
    		std::cerr <<__FILE__<<" "<<__LINE__<< " Error working with config!"<< std::endl; 
    		return -1;
    	}
	try{	
		//! Сливает локальную базу с серверной
		pthread_spin_lock(&lock);
		db.lock();
		neon_operation = true;
		vector <sync_object> srv_files = parseServerTree(&dav);
		neon_operation = false;
		db.combine(srv_files);
		db.unlock();
		//! Обновляет каждый объект синхронизации локальной базы
		sync_object* i=NULL;
         	//! Посылает сообщения о начале синхронизации
        	ipc->send(PSYNCNOTIFY_SYNC_START, PSYNCNOTIFY_IPC_NAME);
	        ipc->send(PSYNCCONFIG_SYNC_START, PSYNCCONFIG_IPC_NAME);
		while(db.next(i)){
			if(conn_ctx.stopflag)
				throw http_exception::user_stop();
			try{
				if(!i->level())
					i->update(&dav);
			}catch(http_exception::error err){
				std::cerr<<"File sync error: "<<err.status()<<std::endl;
				ostringstream msg;
				msg<<err.status();
				ipc->send(PSYNCNOTIFY_HTTP_ERROR, PSYNCNOTIFY_IPC_NAME, msg.str());
			}catch(http_exception::no_space){}
		}
		db.lock();
		db.delDeleted();
		db.save();
		pthread_spin_unlock(&lock);
		db.unlock();
		std::cerr<<"Sync success"<<std::endl;
        //! Посылает сообщения о завершении синхронизации

                cfg.writeInt("cfg.user.lastsync",time(NULL));
		long long max, cur;
		//! Получает квоту пользователя
		if(!dav.GetQuote(max, cur)){
         		cfg.writeLongInt("cfg.info.quota_total", max);
    	        cfg.writeLongInt("cfg.info.quota_used", cur);
		}
                ipc->send(PSYNCNOTIFY_SYNC_STOP, PSYNCNOTIFY_IPC_NAME);
                ipc->send(PSYNCCONFIG_SYNC_STOP, PSYNCCONFIG_IPC_NAME);
        if(cfg.writeConfig()!=0)	std::cerr <<__FILE__<<" "<<__LINE__<< " Error working with config!"<< std::endl;
	}catch(http_exception::user_stop){
		ipc->send(PSYNCNOTIFY_USER_STOP, PSYNCNOTIFY_IPC_NAME);
		ipc->send(PSYNCNOTIFY_SYNC_STOP, PSYNCCONFIG_IPC_NAME);
		db.unlock();
		pthread_spin_unlock(&lock);
	}catch(string str){
		std::cerr<<"Sync thread critical error: "<<str<<std::endl;
		ostringstream msg;
		msg<<str;
/*		if(!conn_ctx.stopflag){
		    ipc->send(PSYNCNOTIFY_HTTP_ERROR, PSYNCNOTIFY_IPC_NAME);
		    ipc->send(PSYNCNOTIFY_HTTP_ERROR, PSYNCCONFIG_IPC_NAME);
		}
		else*/
		if(conn_ctx.stopflag) {
		    ipc->send(PSYNCNOTIFY_USER_STOP, PSYNCNOTIFY_IPC_NAME);
		    ipc->send(PSYNCNOTIFY_SYNC_STOP, PSYNCCONFIG_IPC_NAME);
		}
		db.unlock();
		pthread_spin_unlock(&lock);
	}catch(const char* str){
		std::cerr<<"Sync thread critical error: "<<str<<std::endl;
		ostringstream msg;
		msg<<str;
/*		if(!conn_ctx.stopflag){
		    ipc->send(PSYNCNOTIFY_HTTP_ERROR, PSYNCNOTIFY_IPC_NAME);
		    ipc->send(PSYNCNOTIFY_HTTP_ERROR, PSYNCCONFIG_IPC_NAME);
		}
		else*/
		if(conn_ctx.stopflag) {
		    ipc->send(PSYNCNOTIFY_USER_STOP, PSYNCNOTIFY_IPC_NAME);
		    ipc->send(PSYNCNOTIFY_SYNC_STOP, PSYNCCONFIG_IPC_NAME);
		}
		db.unlock();
		pthread_spin_unlock(&lock);
	}catch(http_exception::auth_err){		//Если 401 - мочим логин и пароль
        cfg.writeString("cfg.user.login", "");
        cfg.writeString("cfg.user.password", "");
        if(cfg.writeConfig()!=0)	std::cerr <<__FILE__<<" "<<__LINE__<< " Error working with config!"<< std::endl;
		mode=MODE_NONE;
		db.unlock();
		pthread_spin_unlock(&lock);				
	}catch(http_exception::moved){					// Меняем сервер
		db.unlock();
		pthread_spin_unlock(&lock);				
        cfg.writeString("cfg.server.address", conn_ctx.host);
        if(cfg.writeConfig()!=0)  	std::cerr <<__FILE__<<" "<<__LINE__<< " Error working with config!"<< std::endl;
        syncAll();						// И перезапускаем
	}catch(http_exception::error err){
		std::cerr<<"Sync thread HTTP critical error: "<<err.status()<<std::endl;
		db.unlock();
		pthread_spin_unlock(&lock);
        }/*catch(...){
		std::cerr << "Error" << std::endl;
        }*/
	startAuto();
	return 0;
}

//! Функция проверки аутенфикации пользователя
/**
 * При ошибке выкидывает исключение
 */
int syncronizer::TryAuth()
{
	try{
		return WebDav().TryAuth();
	}catch(http_exception::moved){
		return WebDav().TryAuth();
	}
}

//! Подключает к классу БД
/**
 * \param path путь к БД
 */
int syncronizer::dbconnect(string path)
{
	db.dbset(path);
	db.load();
	return 0;
}
//! Добавление файла в синхронизацию
/**
 * \param path путь к файлу
 */
int syncronizer::addFile(string path)
{
 	db.insert(path);
	return 0;
}

//! Удаление файла из синхронизации
/**
 * \param path путь к файлу
 */
int syncronizer::removeFile(string path)
{
	sync_object* s;
	s = db.find(path);
	if(!s->isSyncronized())
		db.erase(s);
	else
 		db.remove(path);
	return 0;
}
//! Проверка, в синхронизации ли файл
/**
 * \param path путь к файлу
 * \return 1 - в синхронизации, 0 - нет
 */
int syncronizer::checkFile(string path)
{
	if(db.find(path))
		return 1;
	else 
		return 0;
}
//! Получение синхронизируемых файлов в папке path
/**
 * \param path путь к папке
 * \return список файлов, разделенный \n, или если папка сама в синхронизации - "*"+path
 */
string syncronizer::checkDir(string path)
{
	stringstream ret;
	sync_object* i=NULL;
	path+="/";
	while(db.next(i)){
		string tmp=i->getPath()+"/";
		if((path.find(tmp) == 0) && !i->toDelete())
			return string("*")+path;
		if(i->inDir(path) && !i->toDelete()) 
			ret<<i->getPath()<<'\n';
	}
	return ret.str();
}
//! Получение списка всех синхронизируемых файлов
/**
 * \return список объектов синхронизации первого уровня, разделенный "\n"
 */
string syncronizer::listAll()
{
	stringstream ret;
	sync_object* i=NULL;
	while(db.next(i)){
		if( !(i->toDelete()) && !(i->level()) ) {
			ret<<i->getPath()<<'\n';
		}
	}
	return ret.str();
}

//! Проверка, выполняется ли синхронизации сейчас
/**
 * \return 1 - да, 0- нет
 */
int syncronizer::is_work()
{
	int ret=pthread_spin_trylock(&lock);
	if(!ret)
	{
		pthread_spin_unlock(&lock);
	}
	return !!ret;
}
//! Установка периода синхронизации
/**
 * \param sec период синхронизации в секундах
 */
void syncronizer::setSyncTime(int sec)
{
	unsigned lasttm=0;
	string path_cfg = conn_ctx.home+"sync.cfg";
        Lconfig cfg(path_cfg); //config object

        if(cfg.readConfig()!=0) {
    		std::cerr <<__FILE__<<" "<<__LINE__<< " Error working with config!"<< std::endl; 
    		return;
    	}

        lasttm = cfg.readInt("cfg.user.lastsync"); 
	unsigned currtm=time(NULL);

	sync_time=sec;
	def_synctime=sec;
	if(sec==0) return; // zero parameter from config - sync is off

	if(lasttm==0)
	{
		sync_time = sec;
		cerr<<"timer!: very first sync, set to "<<sync_time<<endl;
		return;
	}

	if(lasttm>=currtm) // something strange -do it now = update last sync time (TODO: think, think, think!)
	{
		cerr<<"timer!: hello future !! well proceed in 1 sec"<<endl;
		sync_time=1;
		return;
	}

	if((currtm-lasttm) >= (unsigned)sync_time) // period already elapsed since last sync time - start sync after 1 sec pause
	{
		cerr<<"timer!: already must do it, wait 1 sec and proceed!"<<endl;
		sync_time=1;
		return;
	}

	sync_time-=(currtm-lasttm);  // just take into account elapsed time
	cerr<<"timer!: set to "<<sync_time<<endl;
}
//! Немедленное выполнение синхронизации
void syncronizer::syncAll()
{
	sync_time=-1;
}
//! Установка режима работы
/**
 * \param _mode режим работы
 */
void syncronizer::setMode(int _mode)
{
	stopAuto();
	mode=_mode;
	startAuto();
}
//! Запуск потока периодической синхронизации
int syncronizer::start()
{
	if(is_working)
		return -1;

        pthread_spin_init(&lock,0);

	is_working=1;
	pthread_create(&th, NULL, ::thread, this);
	return 0;
}
//! Остановка синхронизации
void syncronizer::syncStop()
{
	conn_ctx.stopflag=1;
}
//! Остановка потока периодической синхронизации
void syncronizer::stop()
{
	is_working=0;
//	pthread_join(th,NULL);

        pthread_spin_destroy(&lock);
	pthread_cancel(th);
}
//! Запуск потока автоматической синхронизации
void syncronizer::startAuto()
{
//	if(mode!=MODE_AUTO || sync_time == 0 )
	if(mode!=MODE_AUTO)  return;
	automode=1;
	pthread_create(&ath, NULL, ::athread, this);
}
//! Поток автоматической синхронизации - член класса
void syncronizer::athread()
{
	int ifd;
	int pending;
	conn_ctx.stopflag=0;
	try{	
		//! Инициализирует inotify
		ifd= inotify_init1(IN_NONBLOCK);
		if(ifd==-1)
			throw string("Ошибка получения дескриптора inotify");
		sync_object* i=NULL;
		WebDav* t_dav=new WebDav();

                if(file_operation) {
		     ipc->send(PSYNCNOTIFY_SYNC_START, PSYNCNOTIFY_IPC_NAME);
		     ipc->send(PSYNCCONFIG_SYNC_START, PSYNCCONFIG_IPC_NAME);
		}
		//! Синхронизирует каждый файл и устанавливает на него inotify watch
		while(db.nextall(i)){

			if(file_operation) {
			    try { i->update(t_dav); }
			    catch(...) {}
			}
			if(!i->toDelete()) {
			      int watch = inotify_add_watch(ifd, i->getPath().c_str(), IN_CREATE|IN_DELETE|IN_DELETE_SELF|IN_MODIFY );
			      if(watch == -1)
			   	    cerr<<string("Ошибка добавления файла к inotify")<<i->getPath();
			      else
				    i->set_ifd(watch);
			}
		}
		db.delDeleted();
		delete t_dav;

                if(file_operation) {
  		    ipc->send(PSYNCNOTIFY_SYNC_STOP, PSYNCNOTIFY_IPC_NAME);
		    ipc->send(PSYNCCONFIG_SYNC_STOP, PSYNCCONFIG_IPC_NAME);
		    file_operation = false;
		}

		//! В цикле синхронизирует изменившиеся файлы
		while(automode)
		{
			if(ioctl(ifd, FIONREAD,&pending)!=0)
				throw string("Ошибка ioctl FIONREAD");
			if(!pending){
				poll(0,0,100);
				continue;
			}
			char wbuf[pending];
			int cnt;
			cnt=read(ifd, wbuf, pending);
			if(cnt!=pending)
				throw string ("Ошибка чтения из inotify дескриптора");
			cnt=0;
			WebDav dav;

//cout<<"Auto sync start!\n";
    			ipc->send(PSYNCNOTIFY_SYNC_START, PSYNCNOTIFY_IPC_NAME);
             	        ipc->send(PSYNCCONFIG_SYNC_START, PSYNCCONFIG_IPC_NAME);

			while(pending > 0)
			{
				inotify_event* ie=(inotify_event*)(((char*)wbuf)+cnt);
				cnt+=sizeof(inotify_event)+ie->len;
				pending-=sizeof(inotify_event)+ie->len;
				sync_object* obj=db.find_wd(ie->wd);
				if(!obj){
					inotify_rm_watch(ifd, ie->wd);
					continue;
				}
				try{
					inotify_rm_watch(ifd, ie->wd);
					obj->update(&dav);
					obj->set_ifd( inotify_add_watch(ifd, obj->getPath().c_str(), IN_CREATE|IN_DELETE|IN_DELETE_SELF|IN_MODIFY ));
				}catch(http_exception::error err){
					std::cerr<<"File sync error: "<<err.status()<<std::endl;
					ostringstream msg;
					msg<<err.status();
					ipc->send(PSYNCNOTIFY_HTTP_ERROR, PSYNCNOTIFY_IPC_NAME, msg.str());
				}catch(http_exception::user_stop){
					conn_ctx.stopflag=0;
					ipc->send(PSYNCNOTIFY_USER_STOP, PSYNCNOTIFY_IPC_NAME);
					break;
				}catch(http_exception::no_space){}
//				if(obj->canDelete())
//					db.erase(obj);
				db.delDeleted();
			}
//cout<<"Auto sync stop!\n";
			ipc->send(PSYNCNOTIFY_SYNC_STOP, PSYNCNOTIFY_IPC_NAME);
			ipc->send(PSYNCCONFIG_SYNC_STOP, PSYNCCONFIG_IPC_NAME);

                        pending = 1;
                        while(pending) {
                               ioctl(ifd, FIONREAD, &pending);
                               char nbuf[pending];
                               cnt=read(ifd, nbuf, pending);
                        }
		}
		close(ifd);
	}catch(string str)
	{
		close(ifd);
		automode=0;
		std::cerr<<"Autosync thread critical error: "<<str<<std::endl;
		return ;
	}
	return ;
}
//! Остановка потока автоматической синхронизации
void syncronizer::stopAuto()
{
	if(!automode)
		return;
	automode=0;
	pthread_join(ath,NULL);
}

