//! \file sync_obj.cc реализация класса объекта синхронизации 
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <sstream>
#include <map>
#include <iostream>
#include <iomanip>
#include <errno.h>
#include <cstdlib>
#include <openssl/md5.h>
#include <sys/statfs.h>
#include <time.h>
#include "sync_obj.h"
#include "http_hdr.h"
#include "WebDAV.h"
#include "filesdb.h"
//! Коструктор класса
/**
 * \param _path локальный путь к файлу
 * \param _uname путь на сервере
 */
sync_object::sync_object(string _path,string _uname):path(_path),sync_time(0)
{
//	uname=path.substr(path.rfind("/")+1);
//	uname=_uname+uname;
	uname=path.substr(1);	//Убираем первый '/'
	sync_point=-1;
	struct stat st;
	if(stat(path.c_str(), &st)!=-1){
		mode=st.st_mode;
		if(S_ISDIR(mode))
			uname+="/";
	}else{
		mode=0;
	};//TODO: Сделать чтонибудь если файла нет
	flags=0;
	db=NULL;
	state=SO_UPTODATE;
}
//! Констуктор для инициализации из серверных свойств всех членов класса
/**
 * \param _path локальный путь к файлу
 * \param _uname путь на сервере
 * \param _sync_time время синхронизации
 * \param _stag  sTag на сервере
 * \param _mode  права на файл
 * \param _sync_point точка синхронизации
 * \param _flags флаги объекта
 * \param serversize размер файла на сервере
 */
sync_object::sync_object(string _uname, string _path,time_t _sync_time, string _stag,
		short _mode, char _sync_point, char _flags, size_t  serversize):uname(_uname),
	path(_path),sTag(_stag),sync_time(_sync_time),
		mode(_mode),sync_point(_sync_point),flags(_flags), server_size(serversize)
{ 
	db=NULL;
	state=SO_UPTODATE;
}
//! Функция помощник получения строки из stringstream
/**
 * \param _s ссылка на stringstream
 * \return объект string
 */
static string read_string(stringstream& _s)
{
	unsigned sz;
	_s.read((char*)&sz,sizeof(sz));
	char buf[sz];
	_s.read(buf,sz);
	return string(buf,sz);
}

//! Констуктор для инициализации из БД всех членов класса
/**
 * \param _l строка из БД
 */
sync_object::sync_object(string _l)
{
	stringstream s(_l);

	path=read_string(s);
	uname=read_string(s);
	sTag=read_string(s);
	s.read((char*)&sync_time,sizeof(sync_time));	
	s.read((char*)&mode,sizeof(mode));	
	s.read((char*)&flags,sizeof(flags));	
	s.read((char*)&sync_point,sizeof(sync_point)); 		
	db=NULL;
	state=SO_UPTODATE;

}
//! Конструктор по умолчанию
sync_object::sync_object()
{
	db=NULL;
	state=SO_UPTODATE;
}
//! Деструктор класса
/**
 * Уничтожает за собой всех потомков в ФС
 */
sync_object::~sync_object()
{
	if(!db)
		return;
	if(!(flags&FL_DELETED))
		return;
	if(!S_ISDIR(mode))
		return;
/*
	//Утаскиваем за собой потомков:)
	vector<sync_object*> sl=db->findDir(path);
	for(vector<sync_object*>::iterator i=sl.begin();i!=sl.end();++i){
		db->erase(*i);
	}
*/
}

//! Получение времени изменения файла объекта синхронизации 
/**
 * \return время изменения файла
 */
time_t sync_object::GetMTime()
{
	if(S_ISDIR(mode)){
		time_t stor=0;
		DIR* dir;
		dirent* idir;
		dir=opendir(path.c_str());
		if(dir == NULL)
			return 0;
		while((idir = readdir(dir)) != NULL ){
			if((string(idir->d_name)==".") || (string(idir->d_name) == "..") )
				continue;
			sync_object* nfile=db->find(path+"/"+string(idir->d_name));
			time_t tm;
			if(!nfile){
				struct stat st;
				int ret;
				ret=stat((path+"/"+string(idir->d_name)).c_str(),&st);
				if(ret<0)
					return 0;
				tm= st.st_mtime>st.st_ctime?st.st_mtime:st.st_ctime;
			}
			else 
				tm=nfile->GetMTime();
			stor=stor>tm?stor:tm;
		}
		closedir(dir);
		return stor;
	}else{
		struct stat st;
		int ret;
		ret=stat(path.c_str(),&st);
		if(ret<0)
			return ret;
		return st.st_mtime>st.st_ctime?st.st_mtime:st.st_ctime;
	}
}

//! сравнение объекта со строкой имени файла
/**
 * \param s имя файла
 */
int sync_object::operator==(string s)
{
	return path==s;
}
//! сравнение объекта с другим объектом
/**
 * \param s объект синхронизации
 */
int sync_object::operator==(sync_object s)
{
	return path==s.path;
}
//! Находится ли объект в директории s
/**
 * \param s путь к директории
 */
bool sync_object::inDir(string s)
{
	if(s==path)
		return 0;
        return (path.find(s) == 0) && (path.find("/",s.length()-1) != string::npos);
}
//! Генерация нового sTag'а у объекта синхронизации
void sync_object::genSTag()
{
	ostringstream s,s1;
	string tmp;
	s<<time(NULL);
	s<<rand();
	tmp = string(
		(char*)MD5( (const unsigned char*)s.str().c_str(), s.str().length() ,NULL),
		MD5_DIGEST_LENGTH);
	s1<<uppercase<<hex<<setw(2)<<setfill('0');

	for(unsigned  i=0; i<tmp.length(); i++)
		s1<<(unsigned int)((unsigned char)tmp[i]);
	sTag=s1.str();
}
//! Проверка и синхронизация всех нижестоящих объектов к текущему
/**
 * \param dav - указатель на соединение WebDav
 */
int sync_object::checkSubdir(WebDav* dav)	//TODO: Принимать решение о удалении файлов локально/на сервере на основе state
{
	//Получаем список в папке
	DIR* dir;
	dirent* idir;
	dir=opendir(path.c_str());
//cout<<"Dir for checked - "<<path<<"\n";
        if(dir == NULL)  //смотрим в папке по умолчанию
        {
           cerr<<"Dir "<<path<<" is not open\n";
        }
	else
	{

          sync_object* s = getUpDir();
	  if(s!=NULL) { // && !s->toDelete()
	        if(!level()) leveldown();
	  }
	  if(!toDelete()) {
		while((idir = readdir(dir)) != NULL ){
	  		if((string(idir->d_name)==".") || (string(idir->d_name) == ".."))
				    continue;
			db->insert(path+"/"+string(idir->d_name))->leveldown();
		}
	  }
	  closedir(dir);
	}
	//Получаем список на сервере
	vector<sync_object> list=dav->GetServerList(uname);
	db->combine(list);

	//Получаем список в базе
	vector<sync_object*> sl=db->findDir(path);
	for(vector<sync_object*>::iterator i=sl.begin();i!=sl.end();i++){
		try{
			(*i)->update(dav);
		}catch(http_exception::no_space){
			sync_time=0;
		}
	}
	if(sync_time)
		sync_time=GetMTime();
	return 0;
}

//! Отправка объекта на сервер 
/**
 * \param dav - указатель на соединение WebDav
 */
int sync_object::put(WebDav* dav)
{
	try{
		dav->Lock(uname);
		if(S_ISDIR(mode)){
			dav->Mkcol(uname);
			isCheckSubDir = true;
		}else{
			if(dav->Put(uname,path)){
				dav->Delete(uname);
				dav->Unlock(uname);
				return -1;
			}
		}
		genSTag();
		dav->SetMeta(uname,getMetaMap());
	}catch(...){

		try{
			dav->Delete(uname);
		}catch(...){}
		conn_ctx.ipc->send(PSYNCNOTIFY_FILESEND_ERROR, PSYNCNOTIFY_IPC_NAME, path);
		dav->Unlock(uname);
		throw;
	}
	sync_time=GetMTime();
	dav->Unlock(uname);
	return 0;
}
//! Создание директории до пути path
/**
 *  \param path путь до которого создаются директории
 */
void sync_object::mkdirto(string path)
{
	string p1=path.substr(0, path.rfind("/")+1);
	string::size_type i=0;
	while(true){
		i=p1.find("/",i+1);
		if(i==string::npos)
			break;
		string p2=p1.substr(0,i);
		int ret=mkdir(p2.c_str(),0755);
		if( ret == 0)
			continue;
		if(errno!= EEXIST)
			break;
	}
}

//! Получение объекта с сервера
/**
 * \param dav - указатель на соединение WebDav
 */
int sync_object::get(WebDav* dav)
{
	mkdirto(path);

	sync_object* s = getUpDir();
	if(s!=NULL) {
	    if(!level()) leveldown();
	    if(!s->isCheckSubDir) {
	            isCheckSubDir = false;
		    return 0;
	    }
	}

	try{
		dav->Lock(uname);
		if(S_ISDIR(mode))
		{
		    isCheckSubDir = true;
			int res = mkdir(path.c_str(), mode);
			if(errno == ENOSPC) throw http_exception::no_space();
            if(res!=0 && errno != EEXIST)
            {
                conn_ctx.file_conflict_flag = true;
			    cerr<<"Ошибка создания директории "<< path<<" Ждем решения пользователя.\n";
			    ostringstream msg;
        		msg<<path;
			    conn_ctx.ipc->send(PSYNCNOTIFY_CONFLICT_SAVE_FILE, PSYNCNOTIFY_IPC_NAME, msg.str());
                            conn_ctx.ipc->send(PSYNCNOTIFY_CONFLICT_SAVE_FILE, PSYNCCONFIG_IPC_NAME, msg.str());

	            int t = conn_ctx.timeout + 5;
			    while(t && conn_ctx.file_conflict_flag)
			    {   sleep(1);t--;  }

    			while(conn_ctx.file_wait_flag)    sleep(1);

			    if(conn_ctx.file_path != "0") {
			    	string npath(conn_ctx.file_path);
			    	npath+="/";
			    	npath+=path.substr(path.rfind("/"));
			    	res = mkdir(npath.c_str(), mode);
			    	if ( res!=0 && errno != EEXIST) {
			    		cerr<<"Ошибка создания директории "<< npath;
			    		conn_ctx.ipc->send(PSYNCNOTIFY_FILEDIALOG, PSYNCNOTIFY_IPC_NAME, msg.str());
			    	}
			    	conn_ctx.path_rename = true;
			    	conn_ctx.file_path += path.substr(path.rfind("/"));
			    }
			    else {
	    			conn_ctx.ipc->send(PSYNCNOTIFY_FILEDIALOG, PSYNCNOTIFY_IPC_NAME, msg.str());
	    			isCheckSubDir = false;
	    		}
            }
		}
		else{
			{
				//Size check
				struct statfs sfs;
				int ret=statfs(path.c_str(),&sfs);
				if(ret)
					ret=statfs(path.substr(0,path.rfind("/")).c_str(),&sfs);
				if(!ret){
					if((server_size/sfs.f_bsize) >= sfs.f_bfree)	//Size ERROR
						throw http_exception::no_space();
				}
			}
			//dav->Get(uname,path);
			int res=dav->Get(uname,path);
			if(res==-2) throw http_exception::no_space();
			chmod(path.c_str(),mode);
		}
	}
	catch(http_exception::no_space){
		dav->Unlock(uname);
		unlink(path.c_str());
		conn_ctx.ipc->send(PSYNCNOTIFY_FILERECV_NOSPACE, PSYNCNOTIFY_IPC_NAME, path);
		throw;
	}catch(...){
		dav->Unlock(uname);
		unlink(path.c_str());
		conn_ctx.ipc->send(PSYNCNOTIFY_FILERECV_ERROR, PSYNCNOTIFY_IPC_NAME, path);
		throw;
	}

	if(conn_ctx.path_rename) {
		string old_path = path;

		path = conn_ctx.file_path;
		conn_ctx.path_rename = false;

		string new_path = path;
		replacePath(old_path, new_path);
	}
	sync_time=GetMTime();
	dav->Unlock(uname);
	return 0;
}

//! Замена пути у всех нижестоящих объектов
void sync_object::replacePath(string old_path, string new_path)
{
	string o_path, n_path;
        vector<sync_object*> sl=db->findDir(old_path);
	for(vector<sync_object*>::iterator i=sl.begin();i!=sl.end();i++){
		o_path = (*i)->getPath();
//cout<<"Old path - "<<o_path<<"\n";
		n_path = new_path + o_path.substr(o_path.rfind("/"));
//cout<<"New path - "<<n_path<<"\n";
		(*i)->setPath(n_path);
		(*i)->replacePath(o_path, n_path);
	}
}

//! Получение указателя на вышестоящий объект синхронизации к текущему
sync_object* sync_object::getUpDir()
{
	sync_object* ret;
	ret=db->find(path.substr(0,path.rfind("/")));
	if(!ret) return NULL;
//		throw "Sync object parent search fail";
	return ret;
}

//! Определение статуса объекта(направления и необходимости синхронизации)
/**
 * \param dav - указатель на соединение WebDav
 */
void sync_object::checkState(WebDav* dav)
{
	if(flags&FL_DELETE){
		state=SO_DELETE;
		return;
	}
	if(state!=SO_UPTODATE && 
			state!=SO_CONFLICT)
		return;
	sync_object srv=dav->GetServerState(uname);
	struct stat st;
	if((stat(path.c_str(),&st) && errno == ENOENT) || (stat(path.c_str(),&st) && errno == EACCES)){//Нет у нас
		if(!srv.valid()){
			//		state=SO_CONFLICT;	//TODO:delete
				ostringstream msg;
			int pt=CONF_DEL_ALLSIDE;
			msg.write((char*)&pt,sizeof(int));
			msg<<path;

			conn_ctx.conflict_flag=0;
			conn_ctx.ipc->send(PSYNCNOTIFY_CONFLICT , PSYNCNOTIFY_IPC_NAME, msg.str());
                        int t = conn_ctx.timeout + 5;
			while(t && !conn_ctx.conflict_flag){
				sleep(1);t--;
			}
			switch(conn_ctx.conflict_flag){
				case 1:
					state=SO_CONFLICT;
					return ;
				case 2:
					state=SO_DELETE;
					server_size=srv.server_size;
					break;
				default:
					state=SO_DELETE;
			}
			return;

		}
		if(flags&FL_SYNCRONIZED){
		 	if(level() && getUpDir()->state!=SO_LCREATE){
			        state=SO_DELETE;
			}
			else
				state=SO_LCREATE;
		}
		else
			state=SO_LCREATE;
			server_size=srv.server_size;
		return;
	}

	if(!srv.valid()){//Нет на сервере	//TODO: Вопрос - удален ли на сервере или создан у нас?
		if(flags&FL_SYNCRONIZED){
			state=SO_DELETE;
		}
		else 
			state=SO_PUSH;
		return;
	}
	if(srv.sTag!=sTag){
		state=SO_PULL;
		server_size=srv.server_size;
		mode=srv.mode;
		sTag=srv.sTag;
	}
	if(sync_time==0){
		state=SO_PULL;
		server_size=srv.server_size;
		return;
	}
	if(sync_time < GetMTime()){
		if(state!=SO_UPTODATE)
		{
			//TODO: Конфликт
			//state=SO_CONFLICT;

				ostringstream msg;
				int pt=CONF_MOD_ALLSIDE;	
				msg.write((char*)&pt,sizeof(int));
				msg<<path;

				conn_ctx.conflict_flag=0;
				conn_ctx.ipc->send(PSYNCNOTIFY_CONFLICT , PSYNCNOTIFY_IPC_NAME, msg.str());
				//int t=15;
                                int t = conn_ctx.timeout + 5;
				while(t && !conn_ctx.conflict_flag){
					sleep(1);t--;
				}
				switch(conn_ctx.conflict_flag){
					case 1:
						state=SO_PUSH;
						break;
					case 2:
						state=SO_PULL;
						server_size=srv.server_size;
						break;
					default:
						state=SO_PULL;		//TODO: Переписать на действие по умолчанию из конфигов
				}
				return;

			//conn_ctx.ipc->send(ipc_proxy::NOTE_MSG, conn_ctx.pathNotify.c_str(),
			//	(string("Конфликт при обработке файла ")+path).c_str());
		}
		state=SO_PUSH;
	}
	return;
}

//! Расчет статуса объекта и его синхронизация при необходимости
/**
 * \param dav - указатель на соединение WebDav
 */
int sync_object::update(WebDav* dav)
{
	checkState(dav);
	isCheckSubDir = true;
	cerr<<"Обработка "<<uname<<" ";
	//Удаление
	switch(state){
		case SO_DELETE:
			dav->Delete(uname);
			flags|=FL_DELETED;
			cerr<< "удален"<<endl;
			break;
		case SO_PUSH:
			cerr<<"> на сервер"<<endl;
			put(dav);
			break;
		case SO_PULL:
		case SO_LCREATE:
			cerr<<"< на локальную машину"<<endl;
			get(dav);
			break;
		case SO_CREATE:
			cerr<<"создаем на сервере"<<endl;
			put(dav);
			break;
		case SO_CONFLICT:
			cerr<<" conflict"<<endl;
			break;
		default:
			cerr<<" uptodate"<<endl;
			break;
	}
	if(S_ISDIR(mode) /*&& !(flags&FL_DELETE)*/ && isCheckSubDir) {
		checkSubdir(dav);
	}
	if(state!=SO_CONFLICT && !(flags&FL_DELETED)){
		flags|=FL_SYNCRONIZED;
		state=SO_UPTODATE;
	}
	return 1;
}
//! Получение данных объекта синхронизации
/**
 * \param _uname ссылка на переменную пути на сервере
 * \param _path ссылка на переменную локальный пути к файлу
 * \param _sync_time ссылка на переменную времени синхронизации
 * \param _stag  ссылка на переменную sTag'а на сервере
 * \param _mode  ссылка на переменную прав на файл
 * \param _sync_point ссылка на переменную точки синхронизации
 * \param _flags ссылка на переменную флагов объекта
 */
void sync_object::getData(string& _uname, string& _path,time_t& _sync_time,
		string& _stag, mode_t& _mode, char& _sync_point, char& _flags) const
{
	_uname=uname;
	_path=path;
	_sync_time=sync_time;			//Local
	_stag=sTag;
	_mode=mode;
	_sync_point=sync_point;
	_flags=flags;
}
//! статический метод записи string в stringstream
static void write_string(stringstream& _s,const string s)
{
	unsigned sz=s.size();
	_s.write((char*)&sz,sizeof(sz));
	_s.write(s.c_str(),sz);
}
//! Получение всех данных класса в виде строки для сохранения
string sync_object::getStringData() const
{
	stringstream s;
	write_string(s, path);
	write_string(s, uname);
	write_string(s, sTag);
	s.write((char*)&sync_time,sizeof(sync_time));	
	s.write((char*)&mode,sizeof(mode));	
	s.write((char*)&flags,sizeof(flags));	
	s.write((char*)&sync_point,sizeof(sync_point)); 		
	return s.str();
}
//! Получение ассоциативного массива свойств объекта для записи на сервер
map<string,string> sync_object::getMetaMap() const
{
	map<string,string> ret;
	if(!(flags&FL_SYNCRONIZED)) {
		ret["clientpath"]=path;
	}
	ret["getstag"]=sTag;
	ostringstream m;
	m<<mode;
	ret["filemode"]=m.str();
	ostringstream _m;
	_m<<level();
	ret["level"]=_m.str();
	return ret;
}
//! Проверка валидности объекта синхронизации 
/**
 * \return 0 - невалиден, 1 - все нормально
 */
int sync_object::valid()
{
	if(uname.empty())
		 return 0;
        else if(uname[uname.length()-1]!='/') {
                path = "/" + uname;
                if(mode == 0)
                    mode = 33204;
        }
	if(path.empty())
		 return 0; 
	if(mode==0)
		 return 0; 
	return 1;
}

//! Установка флага подготовки для удаления объекта и всех потомков
void sync_object::setDelete()
{
	flags|=FL_DELETE;

	if(!S_ISDIR(mode))
		return;

	//Утаскиваем за собой потомков:)
	vector<sync_object*> sl=db->findDir(path);
	for(vector<sync_object*>::iterator i=sl.begin();i!=sl.end();++i){
		(*i)->setDelete();
	}
}
